package lbl.akenti.gen.controller;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;


/**
  * Allows the user to create a url or a directory.
  *
  * @see javax.swing.JPanel
  */
public class LBLDirField 
  extends JPanel
  implements ActionListener, DocumentListener {
  private final static String DIR_PANEL_TITLE = "Directory Builder";

  private String[] protocols;

  private JTextField dirField;
  private DocumentListener docListener;

  private LBLBuildDirPane optionPane;
  private JButton createButton = new JButton("Create");

  /** 
    * Constructor 
    */
  public LBLDirField() {
    this(null);
  }
  
  /**
    */
  public LBLDirField(String label) {
    this(label, null);
  }

  /** 
    * Constructor 
    */
  public LBLDirField(String label, String []protocols) {
    if (protocols != null) {
      this.protocols = protocols;
    } 

    dirField = LBLTextFieldFactory.createTextField(this);
    createButton.setActionCommand("create");
    createButton.addActionListener(this);
    LBLButtonPanel.enableBlueButton(createButton);

    if (label == null) {
      WrapUtil.wrapHorizontalIn(this, null, dirField, createButton);
      return;
    }

    JLabel temp = new JLabel(label);

    temp.setForeground(Color.red);
    WrapUtil.wrapHorizontalIn(this, temp, dirField, createButton);
  }
  
  /**
    */
  private LBLBuildDirPane getDirOptionPane() {
    if (optionPane == null) {
      optionPane = new LBLBuildDirPane(protocols);
    }

    return optionPane;
  }

  /**
    */
  public void actionPerformed(ActionEvent event) {
    String cmd = event.getActionCommand();

    if (cmd.equals("create")) {
      LBLBuildDirPane optionPane = getDirOptionPane();

      optionPane.clear();
      optionPane.createDialog(this, DIR_PANEL_TITLE).setVisible(true);

      Object test = optionPane.getValue();
      if (test != null && !test.equals(GenConsts.AK_CANCEL_VALUE)) {
	String dir = (String) test;
        dirField.setText(dir);
        optionPane.setValue(GenConsts.AK_CANCEL_VALUE);
      }
    } else { // REMOVE 
      throw new RuntimeException(cmd + "not supported");
    }
  }
  
  /**
    */
  public void setDocumentListener(DocumentListener docListener)  {
    this.docListener = docListener;
  }

  /**
    */
  public void changedUpdate(DocumentEvent e)  {
    if (docListener != null) {
      docListener.changedUpdate(e);
    } 
  }

  /**
    */
  public void insertUpdate(DocumentEvent e)  {
    if (docListener != null) {
      docListener.insertUpdate(e);
    } 
  }

  /**
    */
  public void removeUpdate(DocumentEvent e) {
    if (docListener != null) {
      docListener.removeUpdate(e);
    } 
  }

  /**
    */
  void setDir(String dir) {
    dirField.setText(dir);
  }

  /**
    */
  String getDir() {
    return dirField.getText();
  }

  /**
    */
  public void clear() {
    setDir("");
  }

  /**
    */
  private final static String DIALOG_TITLE =  "URL BUILDER";

  /**
    */
  private static String DIALOG_INFO = "Build a directory where we can load the ca's x509 certs";

  /**
    */
  public static String 
  showDialog(JComponent owner) {
    LBLDirField dirField = new LBLDirField("Directory");

    while (true) {
      boolean ret = LBLDialogFactory.showDialog(
        owner, 
        DIALOG_TITLE,
        DIALOG_INFO,
        dirField
      );

      if (!ret) {  // CANCEL
        return GenConsts.AK_CANCEL_VALUE.toString();
      }
      
      try {
        URL url = new URL(dirField.getDir());

        return dirField.getDir();
      } catch(Exception ex) {
        // XXX 
      }
    }
  }
}
