package lbl.akenti.gen.controller;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JComponent;
import javax.swing.Box;
import javax.swing.BoxLayout;
import java.awt.GridBagLayout;
import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;

/**
  * A utility class to wrap components.
  */

public class WrapUtil {
  
  /**
    */
  private WrapUtil() {
  }

  /**
    */
  public static JPanel wrapHorizontal(JLabel label, JTextField textField) {
    return wrapHorizontal(label, textField, null);
  }

  /**
    */
  public static JPanel wrapHorizontal(JTextField textField, JButton button) {
    return wrapHorizontal(null, textField, button);
  }

  /**
    */
  public static JPanel wrapHorizontal(JLabel label, JTextField textField, JButton button) {
    JPanel panel = new JPanel();

    wrapHorizontalIn(panel, label, textField, button);
    return panel;
  }

  public static void 
  wrapHorizontal2In(JComponent panel, JLabel label, JTextField textField, JButton button) {
    GridBagLayout gridbag = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();

    panel.setLayout(gridbag);
    c.anchor = GridBagConstraints.EAST;
    c.insets = new Insets(5, 5, 5, 5);

    c.fill = GridBagConstraints.NONE;      
    c.weightx = 0.0;                     
    gridbag.setConstraints(label, c);
    panel.add(label);

    c.fill = GridBagConstraints.HORIZONTAL;
    c.weightx = 1.0;
    gridbag.setConstraints(textField, c);
    panel.add(textField);

    c.gridwidth = GridBagConstraints.REMAINDER;
    c.fill = GridBagConstraints.NONE;
    c.weightx = 0.0;
    gridbag.setConstraints(button, c);
    panel.add(button);
  }

  public static void 
  wrapHorizontalIn(JComponent panel, JLabel label, JTextField textField, JButton button) {
    GridBagLayout gridbag = new GridBagLayout();

    panel.setLayout(gridbag);
    GridBagConstraints c = new GridBagConstraints();

    c.anchor = GridBagConstraints.EAST;
    c.insets = new Insets(5, 5, 5, 5);

    if (label != null) {
      c.fill = GridBagConstraints.NONE;      
      c.weightx = 0.0;                     
      gridbag.setConstraints(label, c);
      panel.add(label);
    }

    if (button == null) {
      c.gridwidth = GridBagConstraints.REMAINDER;
    } 

    c.weightx = 1.0;
    c.fill = GridBagConstraints.HORIZONTAL;
    gridbag.setConstraints(textField, c);
    panel.add(textField);

    if (button != null) {
      c.gridwidth = GridBagConstraints.REMAINDER;
      c.fill = GridBagConstraints.NONE;
      c.weightx = 0.0;
      gridbag.setConstraints(button, c);
      panel.add(button);
    }
  }

  /**
    */
  public static JPanel wrapHorizontal(JLabel label, JComponent comp) {
    return wrapComponents(new JLabel [] { label }, new JComponent[] { comp });
  }
  
  /**
    */
  public static JPanel wrapHorizontal(JLabel []labels, JComponent []components) {
    return wrapComponents(labels, components);
  }

  /**
    */
  public static JPanel wrapComponents(JComponent []labels, JComponent []components) {
    GridBagLayout gridbag = new GridBagLayout();
    JPanel panel = new JPanel(gridbag);
    GridBagConstraints c = new GridBagConstraints();

    c.anchor = GridBagConstraints.EAST;
    c.insets = new Insets(5, 5, 5, 5);
    
    for (int i = 0; i < labels.length; i++) {
      c.gridwidth = GridBagConstraints.RELATIVE; 
      c.fill = GridBagConstraints.NONE;      
      c.weightx = 0.0;                     
      gridbag.setConstraints(labels[i], c);
      panel.add(labels[i]);

      c.gridwidth = GridBagConstraints.REMAINDER;
      c.fill = GridBagConstraints.HORIZONTAL;
      c.weightx = 1.0;
      gridbag.setConstraints(components[i], c);
      panel.add(components[i]);
    }

    return panel;
  }

  /**
    *  Wraps two components horizontally next to each other 
    */
  public static JPanel wrapComponents(JComponent comp1, JComponent comp2) {
    // return wrapComponents(new JComponent[] { comp1 }, new JComponent[] { comp2 });

    JPanel panel = new JPanel();

    panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
    panel.add(comp1);
    panel.add(Box.createHorizontalStrut(5));
    panel.add(comp2);
    return panel;
  }

  /** 
    * wraps two components vertically on top of each other 
    */
  public static JPanel wrapVertical(JComponent comp1, JComponent comp2) {
    return wrapVertical(new JComponent[] { comp1, comp2 });
  }

  /**
    */
  public static JPanel wrapVertical(JComponent []comps) {
    JPanel panel = new JPanel();

    panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
    panel.add(comps[0]);
    
    for (int i = 1; i < comps.length; ++i) {
      panel.add(Box.createVerticalStrut(5));
      panel.add(comps[i]);
    }

    return panel;
  }
}

