package lbl.akenti.gen.controller;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Component;
import java.io.File;
import java.net.URL;
import java.net.MalformedURLException;
import java.security.cert.X509Certificate;

import lbl.akenti.util.X509CertReader;
import lbl.akenti.gen.common.GenCa;
import lbl.akenti.certs.elements.Directory;

/**
  */
public class X509CertRetrieval {

  private LBLBuildDirPane buildDirPane;
  private LBLLdapPanel ldapPanel;
  private JFileChooser chooser;
  private Component parent;

  /**
    */
  public X509CertRetrieval(Component parent) {
    if (parent == null) {
      throw new IllegalArgumentException();
    }

    this.parent = parent;
  }

  private final static String BUILD_DIR_PANEL_MESSAGE =
    "<html><font size=+1 color=blue>Looking for X509 certificates..." +
    "<li><font color=blue>Enter the url at which the CA publishes certificates." +
    "<li><font color=blue>If <i>ldap</i> and <i>http</i> the base is used to start the search." +
    "<li><font color=blue>For file protocol the base is the full path name." +
    "<li><font color=blue>Clicking Use Hints will step you through the process.";

  /**
    */
  private LBLBuildDirPane getDirOptionPane() {
    if (buildDirPane == null) {
      buildDirPane = new LBLBuildDirPane(BUILD_DIR_PANEL_MESSAGE, null);
      buildDirPane.setOkLabel("Continue");
    }

    return buildDirPane;
  }

  /**
    */
  private LBLLdapPanel getLdapPanel() {
    if (ldapPanel == null) {
      ldapPanel = new LBLLdapPanel("Search for X509 Certificate");
    }

    return ldapPanel;
  }

  private X509Certificate[] loadCertFromLdap(URL url) {
    LBLLdapPanel ldapPanel = getLdapPanel();
    URL prev = ldapPanel.getURL();

    if (prev == null || !prev.equals(url)) {
      ldapPanel.setURL(url);

      if (!ldapPanel.loadOrgUnits()) {
        return null;
      }
    }

    ldapPanel.createDialog(parent, "X509 Certificate Retrieval").setVisible(true);
    Object ret = ldapPanel.getValue();
   
    if (ret == GenConsts.AK_CANCEL_VALUE) {
      return null;
    }

    return (X509Certificate[]) ret;
  }

  /**
    */
  private JFileChooser getFileChooser() {
    if (chooser == null) {
      chooser = new JFileChooser();
    }
     
    return chooser;
  }

  public X509Certificate[] 
  loadX509Certificates(String dir) throws MalformedURLException {
    return loadX509Certificates(new URL(dir));
  }

  /**
    */
  public X509Certificate[] 
  loadX509Certificates(URL url) {
    try {
      if (url.getProtocol().equalsIgnoreCase("ldap")) {
        return loadCertFromLdap(url);
      } else if (url.getProtocol().equalsIgnoreCase("file")) {
        File temp = new File(url.getFile());

        if (temp.isDirectory()) {
          JFileChooser chooser = getFileChooser();

          chooser.setCurrentDirectory(temp);
          chooser.setDialogTitle("Select the file that contains the X509 Certificate");
          chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

          if (chooser.showOpenDialog(parent) != JFileChooser.APPROVE_OPTION) {
            return null;
          } 

          temp = chooser.getSelectedFile();
          return new X509CertReader().readX509Certs(temp.getAbsolutePath());
        }
      } 

      // Handles files and other urls.
      return new X509CertReader().readX509Certs(url);
    } catch(Exception ex) { // XXX
      LBLDialogFactory.showInfoDialog(
        parent,
        "Error",
        "<html><font color=red>Unable to read cert<li><font color=red>" + ex
      );
      
      return null;
    }
  } 
  
  /**
    */
  public X509Certificate[] loadX509Certificates() {
    LBLFrame frame = (LBLFrame) JOptionPane.getFrameForComponent(parent);

    return loadX509Certificates(frame.getController().getGenCas()); 
  }

  /**
    */
  public X509Certificate[] loadX509Certificates(GenCa[] cas) {
    LBLBuildDirPane buildDirPane = getDirOptionPane();
    JDialog dialog = buildDirPane.createDialog(parent, "X509 Certificate Retrieval");

    buildDirPane.setGenCas(cas);

    dialog.pack();

    X509Certificate[] certs = null;

    while (certs == null) {
      dialog.setVisible(true);

	
      Object ret = buildDirPane.getValue();

      if (ret == null || ret == GenConsts.AK_CANCEL_VALUE) {
        return null;
      }

      String dir = (String) ret;

      buildDirPane.setValue(null);

      try {
        certs = loadX509Certificates(dir);
      } catch(MalformedURLException ex) {
        throw new RuntimeException();
      }
    }
    
    return certs;
  }
}
