package lbl.akenti.gen.controller.pol;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListDataEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Color;
import java.awt.BorderLayout;

import lbl.akenti.gen.controller.GenConsts;
import lbl.akenti.gen.controller.LBLButtonPanel;
import lbl.akenti.gen.controller.LBLFrame;

import lbl.akenti.certs.elements.CertificateAuthorityInfo;

/**
  */
public class PolCAInfosFrame
  extends LBLFrame 
  implements ListDataListener, ListSelectionListener, ActionListener {

  private JList caInfoList;

  private JButton viewButton;
  private JButton remButton;
  private boolean optional;

  private CAInfoPanel caInfoPanel;

  /** 
    * Constructs this frame.
    *
    * @param title title of the frame
    * @param instructions message that goes in the top portion of this frame   
    */
  public PolCAInfosFrame(String title, String instructions) {
    super(title, instructions);	

    JPanel temp = new JPanel(new BorderLayout(5, 5));
    JLabel label = new JLabel("Certificate Authorities");

    label.setForeground(Color.red);

    DefaultListModel model = new DefaultListModel();

    model.addListDataListener(this);
    caInfoList = new JList(model);
    caInfoList.setSelectionMode(2);
    caInfoList.addListSelectionListener(this);
    temp.add(wrapVertical(label, new JScrollPane(caInfoList)), "Center");
         
    JPanel buttonPanel = new JPanel();
    JButton addButton = new JButton("Add");
    
    enableBlueButton(addButton);
    addButton.setActionCommand("add");
    addButton.addActionListener(this);
    buttonPanel.add(addButton);

    viewButton = new JButton("View");
    viewButton.addActionListener(this);
    viewButton.setActionCommand("view");
    buttonPanel.add(viewButton);
    
    remButton = new JButton("Remove");
    remButton.addActionListener(this);
    buttonPanel.add(remButton);
    remButton.setActionCommand("remove");
    temp.add(buttonPanel, "South");

    JPanel contentPane = (JPanel) getContentPane();

    contentPane.add(temp, "Center");
    doEnable();
  }

  public CAInfoPanel getCAInfoPanel() {
    if (caInfoPanel == null) {
      caInfoPanel = new CAInfoPanel();
    }
    
    return caInfoPanel;
  }

  /**
    */
  public void actionPerformed(ActionEvent event) {
    String cmd = event.getActionCommand();

    if (cmd.equals("view")) {
      CAInfoPanel caInfoPanel = getCAInfoPanel();
      int index = caInfoList.getSelectedIndex();
      DefaultListModel model = (DefaultListModel) caInfoList.getModel();
      CertificateAuthorityInfo info = (CertificateAuthorityInfo) model.getElementAt(index);

      caInfoPanel.setOKLabel("Apply Changes");
      caInfoPanel.setCAInfo(info);
      caInfoPanel.createDialog(this, "CAInfoViewer").setVisible(true);

      Object val = caInfoPanel.getValue();
      if ((val != null) && !(val.equals(GenConsts.AK_CANCEL_VALUE))) {
        model.removeElementAt(index);
        model.insertElementAt(val, index);
        caInfoList.addSelectionInterval(index, index);
      }
    } else if(cmd.equals("add")){
      CAInfoPanel caInfoPanel = getCAInfoPanel();

      caInfoPanel.clear();
      caInfoPanel.setOKLabel("Add CA Info");
      caInfoPanel.createDialog(this, "CAInfoBuilder").setVisible(true);

      Object val = caInfoPanel.getValue();

      if ((val != null) && !(val.equals(GenConsts.AK_CANCEL_VALUE))) {
        addCAInfo((CertificateAuthorityInfo) val);
      }
    } else if(cmd.equals("remove")){
      DefaultListModel model = (DefaultListModel) caInfoList.getModel();
      int []indices = caInfoList.getSelectedIndices();

      for (int i = indices.length - 1; i > -1; --i) {
        model.removeElementAt(indices[i]);
      }

    } else {
      throw new RuntimeException(cmd + " not supported");
    }
  }

  /**
    * If optional is set to true then the next button
    * should be enabled. 
    *
    * The idea is that this policy certificates 
    * will inherit the ca infos from the parent
    * policy. 
    *
    */
  public void setOptional(boolean optional) {
    this.optional = optional;
    doEnable();
  }

  /**
    */
  public void clear() {
    DefaultListModel model = (DefaultListModel) caInfoList.getModel();

    model.clear();
  }

  /**
    */
  public void valueChanged(ListSelectionEvent e) {
    doEnable();
  }

  /**
    */
  public void contentsChanged(ListDataEvent e) {
    doEnable();
  }

  /**
    */
  public void intervalAdded(ListDataEvent e) {
    doEnable();
  }

  /**
    */
  public void intervalRemoved(ListDataEvent e)  {
    doEnable();
  }

  /**
    */
  private void doEnable() {
    Object[] values = caInfoList.getSelectedValues();

    if (values.length != 0) {
      enableRedButton(remButton);

      if (values.length == 1) {
        enableBlueButton(viewButton);
      } else {
        disableBlueButton(viewButton);
      }
    } else {
      disableBlueButton(viewButton);
      disableRedButton(remButton);
    }
    
    DefaultListModel model = (DefaultListModel) caInfoList.getModel();

    if (!optional && model.getSize() == 0) {
      disableBlueButton("Next");
    } else {
      enableBlueButton("Next");
    }
  }

  /**
    */
  public void setCAInfos(CertificateAuthorityInfo[] infos) {
    clear();
    addCAInfos(infos);
  }

  /**
    */
  public int addCAInfos(CertificateAuthorityInfo[] infos) {
    DefaultListModel model = (DefaultListModel) caInfoList.getModel();
    int added = 0;
    int size = model.getSize();

    caInfoList.clearSelection();

    for (int i = 0; i < infos.length; ++i) {
      int index = model.indexOf(infos[i]);

      if (index != -1) {
        caInfoList.addSelectionInterval(index, index);
      } else {
        model.addElement(infos[i]);
        caInfoList.addSelectionInterval(size, size++);
        ++added;
      }
    }

    return added;
  }

  /**
    */
  public boolean addCAInfo(CertificateAuthorityInfo info) {
    return addCAInfos(new CertificateAuthorityInfo[] { info }) != 0; 
  }

  /**
    */
  public CertificateAuthorityInfo[] getCAInfos() {
    DefaultListModel model = (DefaultListModel) caInfoList.getModel();

    CertificateAuthorityInfo[]infos = new CertificateAuthorityInfo[model.getSize()];
    model.copyInto(infos);
    return infos;
  }
}

