package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiMsgType;

/**
 * Associates a <code>code</code> with a <code>message</code>.
 * @author Abdelilah Essiari
 */

final public 
class AkentiMessage {
  private int code;
  private String message;

  /**
    * Default constructor.
    */
  public AkentiMessage() {
    this(-1, "NO MESSAGE");
  }

 /**
   * Constructs this object.
   * @param code the code
   * @param message the text of the message
   */
  public AkentiMessage(int code, String message) {
    if (message == null) {
      throw new IllegalArgumentException();
    }

    this.code = code;
    this.message = message;
  }

 /**
   * Returns the <code>code</code> associated with <code>AkentiMessage</code>
   * @return the code
   */
  public int getCode() {
    return code;
  }

 /**
   * Returns the <code>message</code> associated with <code>AkentiMessage</code>
   * @return the text of the message
   */
  public String getMessage() {
    return message;
  }

 /**
   * A string consisting of the <code>code</code> followed by a space
   * followed by the <code>message</code>.
   * @return this object in string form
   */
  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append(code);
    sb.append(' ');
    sb.append(message);
    return sb.toString();
  }

 /**
   * Constructs an <code>AkentiMessage</code> from an <code>AkentiMessageType</code>.
   * @param impl the AkentiMessageType
   */
  AkentiMessage(AkentiMsgType impl) {
    this(impl.getCode(), impl.getStatus());
  }

 /**
   * Produces an <code>AkentiMsgType</code> containing the information in
   * this object.
   * @throws JAXBException if mandatory data is not present
   * @return the AkentiMsgType
   */
  AkentiMsgType toJAXBImpl() throws JAXBException {
    AkentiMsgType impl = Util.getObjectFactory().createAkentiMsgType();

    impl.setCode(getCode());
    impl.setStatus(getMessage());
    return impl;
  }
}
