package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiResponseType;

/**
 * Encapsulates the various responses that are recieved from the
 * <i>Akenti Server</i>.
 *
 * Each response contains the client-generated unique id of the
 * corresponding request. That can be useful when clients are
 * sending or keeping track of multiple requests.
 */
abstract public
class AkentiResponse {
  private int id;
  private AkentiMessage mesg;

 /**
   * The constructor.
   * @param id the unique id taken from the corresponding request
   * @param mesg the main akenti message associated to this process.
   */
  protected  AkentiResponse(int id, AkentiMessage mesg) {
    if (id < 0 || mesg == null) {
      throw new IllegalArgumentException();
    }

    this.mesg = mesg;
    this.id = id;
  }

 /**
   * Returns the unique id. It should correspond to the unique id of the request.
   * @return the id
   */
  public int getID() {
    return id;
  }

 /**
   * The AkentiMessage describing the status of the request. Generally an
   * AkentiMessage with a code between 100-200 is a success all other codes
   * indicate a failure of some kind
   * @return the AkentiMessage
   */
  public AkentiMessage getMessage() {
    return mesg;
  }

 /**
   * Produces a AkentiRequestType to encapsulate the data in this object.
   * @throws JAXBException if mandatory data is not present
   * @return the AkentiRequestType
   */
  final AkentiResponseType toJAXBImpl() throws JAXBException {
    AkentiResponseType impl = doToJAXBImpl();
 
    impl.setID(getID());
    impl.setAkentiMessage(getMessage().toJAXBImpl());
    return impl;
  }
 
 /**
   * turns this response into the AkentiResponseType generated by jaxb
   * @throws JAXBException if mandatory data is not present
   * @return the AkentiResponseType
   */
  abstract protected AkentiResponseType doToJAXBImpl() throws JAXBException;

 /**
   * Constructs this AkentiRequests from an AkentiRequestsType.
   * @param impl the AkentiRequestsType
   */
  AkentiResponse(AkentiResponseType impl) {
    id = impl.getID();
    mesg = new AkentiMessage(impl.getAkentiMessage());
  }
}
