package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.CreateResourceRequestType;

/**
 * A simple request to create a (non-root) resource.
 * @author Tommaso Sciortino
 */
final public
class CreateResourceRequest extends AkentiRequest {
  private String resource;

 /**
   * Constructs a <code>CreateResourceRequest</code>.
   * @param uid the id
   * @param resource the resource to be created
   */
  public CreateResourceRequest(int uid, String resource) {
    super(uid);

    if (resource.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.resource = resource;
  }

 /**
   * Returns the resource to be created.
   * @return the resource.
   */
  public String getResource() {
    return resource;
  }

 /**
    * USED FOR DOING XML STUFF 
    */
  CreateResourceRequest(CreateResourceRequestType impl) {
    super(impl);
    resource = impl.getResource();
  }
  
 /**
   * Produces a CreateResourceRequestType to encapsulate the data in this object.
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   * @return the CreateResourceRequestType
   */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    CreateResourceRequestType impl = type? Util.getObjectFactory().createCreateResourceRequestType()
                                           : Util.getObjectFactory().createCreateResourceRequestGenerated();

    impl.setResource(getResource());
    return impl;
  }
}
