package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.DeletePolicyCertificateRequestType;

/**
 * A request to delete a policy certificate.
 * @author Tommaso Sciortino
 */
public final class DeletePolicyCertificateRequest extends AkentiRequest {
  private String resource;

 /**
   * Constructs this <code>DeletePolicyCertificateRequest</code> with a resource.
   * @param uid the id
   * @param resource the resource
   */
  public DeletePolicyCertificateRequest(int uid, String resource) {
    super(uid);

    if (resource.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.resource = resource;
  }

 /**
   * Returns the resource of the Policy to be deleted.
   * @return the resource
   */
  public String getResource() {
    return resource;
  }

 /**
    * USED FOR DOING XML STUFF 
    */
  DeletePolicyCertificateRequest(DeletePolicyCertificateRequestType impl) {
    super(impl);
    resource = impl.getResource();
  }
  
 /**
   * Produces a DeletePolicyCertificateRequestType to encapsulate the data in this object.
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   * @return the DeletePolicyCertificateRequestType
   */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    DeletePolicyCertificateRequestType impl = type? Util.getObjectFactory().createDeletePolicyCertificateRequestType()
                                           : Util.getObjectFactory().createDeletePolicyCertificateRequestGenerated();

    impl.setResource(getResource());
    return impl;
  }
}
