package lbl.akenti.messaging;

import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiResponseType;
import lbl.akenti.jaxb.GetResourceInfoResponseType;

/**
 * A response to a request to return the resource attributes (hints) file.
 * @author Tommaso Sciortino
 */
final public
class GetResourceInfoResponse extends AkentiResponse {
  private String resourceInfo;

 /**
   * Constructs a <code>GetResourceInfoResponse</code>.
   * @param uid the id
   * @param mesg the AkentiMessage
   */
  public GetResourceInfoResponse(int uid, AkentiMessage mesg) {
    this(uid, mesg, null);
  }

 /**
   * Constructs a <code>GetResourceInfoResponse</code> with contents of hints file.
   * @param uid the id
   * @param mesg the AkentiMessage
   * @param resourceInfo contents of hints file as a string
   */
  public GetResourceInfoResponse(int uid, AkentiMessage mesg, String resourceInfo) {
    super(uid, mesg);
    this.resourceInfo = resourceInfo;
  }

 /**
   * Returns the contents of the hint file as a string.
   * @return the contents of the hint file.
   */
  public String getResourceInfo() {
    return resourceInfo;
  }

 /**
    */
  GetResourceInfoResponse(GetResourceInfoResponseType impl) {
    super(impl);

    resourceInfo = impl.getResourceInfo();
  }
  
 /**
   * Produces a GetResourceInfoResponseType to encapsulate the data in this object.
   * @throws JAXBException if mandatory data is not present
   * @return the GetResourceInfoResponseType
   */
  protected AkentiResponseType doToJAXBImpl() throws JAXBException {
    GetResourceInfoResponseType impl =  Util.getObjectFactory().createGetResourceInfoResponseGenerated();

    impl.setResourceInfo(getResourceInfo());
    return impl;
  }
}
