package lbl.akenti.messaging;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.CertificateInfoType;
import lbl.akenti.jaxb.PolicyInfoType;

/**
  * Represents a per-resource immutable non-empty list of 
  * <code>CertificateInfo</code>s.
  *
  * @author Abdelilah Essiari
  */
final public
class PolicyInfo {
  private List certInfosList;
  private boolean policyCorrect;
  private String resource;

 /**
   * Constructs this <code>PolicyInfo</code> with a single
   * <code>CertificateInfo</code>.
   * @param cr a CertificateInfo
   */
  public PolicyInfo(CertificateInfo cr) {
    this(new CertificateInfo[] { cr });
  }

 /**
   * Constructs this <code>PolicyInfo</code> with array
   * <code>CertificateInfo</code> whose size is at least 1.
   * @param crs an array of CertificateInfos
   */
  public PolicyInfo(CertificateInfo[] crs) {
    certInfosList = new ArrayList(crs.length);

    for (int i = 0; i < crs.length; ++i) {
      certInfosList.add(crs[i]);
    } 

    certInfosList = Collections.unmodifiableList(certInfosList);
  }
  
 /**
   * Constructs this <code>PolicyInfo</code> with array
   * <code>CertificateInfo</code> whose size is at least 1.
   * @param cr a CertificateInfo
   * @param resource the resource which this Policy describes
   * @param policyCorrect true if the policy is well-formed and correct
   */
  public PolicyInfo(CertificateInfo cr, String resource, boolean policyCorrect) {
    this(new CertificateInfo[] { cr },resource,policyCorrect);
  }
  
 /**
   * Constructs this <code>PolicyInfo</code> with array
   * <code>CertificateInfo</code> whose size is at least 1.
   * @param crs an array of CertificateInfos
   * @param resource the resource which this Policy describes
   * @param policyCorrect true if the policy is well-formed and correct
   */
  public PolicyInfo(CertificateInfo[] crs, String resource, boolean policyCorrect) {
    certInfosList = new ArrayList(crs.length);

    for (int i = 0; i < crs.length; ++i) {
      certInfosList.add(crs[i]);
    } 

    certInfosList = Collections.unmodifiableList(certInfosList);
    this.resource = resource;
    this.policyCorrect = policyCorrect;
  }

 /**
   * Returns the CertificateInfos as an array.
   * @return the CertificateInfos
   */
  public CertificateInfo[] toCertificateInfosArray() {
    CertificateInfo[] ret = new CertificateInfo[certInfosList.size()];
  
    certInfosList.toArray(ret);
    return ret;
  }

 /**
   * Returns the <code>CertificateInfo</code>s as an iterator.
   * @return the iterator
   */
  public Iterator getCertificateInfos() {
    return certInfosList.iterator();
  }
   
 /**
   * Returns the resource.
   * @return the resource which this Policy describes
   */
  public String getResource() {
    return resource;
  }
    
 /**
   * Returns true if the policy is well-formed and correct.
   * @return true if policy works
   */
  public boolean isPolicyCorrect() {
    return policyCorrect;
  }
  
 /** 
    * JAXB STUFF.
    */
  PolicyInfo(PolicyInfoType impl) 
    throws IOException, ParseException {
    List tempList = impl.getCertificateInfo();
    Iterator iter = tempList.iterator();

    certInfosList = new ArrayList(tempList.size());

    do {
      CertificateInfoType art = (CertificateInfoType) iter.next();

      certInfosList.add(new CertificateInfo(art));
    } while (iter.hasNext());

    certInfosList = Collections.unmodifiableList(certInfosList);
    
    resource = impl.getResource();
  }

 /** 
    * JAXB STUFF.
    */
  PolicyInfoType toJAXBImpl() throws JAXBException {
    PolicyInfoType impl = Util.getObjectFactory().createPolicyInfoType();
    Iterator iter = getCertificateInfos();
    List tempList = impl.getCertificateInfo();

    do {
      CertificateInfo ar = (CertificateInfo) iter.next();

      tempList.add(ar.toJAXBImpl());
    } while (iter.hasNext());

    return impl;
  }
}
