package lbl.akenti.parsers;

/**
  * Subclasses of this class are used to evaluate 
  * simple relational expressions.
  *
  * @author Abdelilah Essiari
  * @see ExpressionEvaluator#evaluate(Expression)
  * @see ExpressionEvaluator#reduce(Expression)
  */
public interface 
Evaluator {

  /**
    * Returns the result of the evaluation of the expression <code>attribute op value</code>.
    *
    * @see TokenType
    */
  boolean evaluate(TokenType op, String attribute, String value);
 
  /**
    * Returns the result of the evaluation of the expression <code>attribute op value</code>.
    * Evaluation can result in yet another expression. Null if not 
    *
    * @return Result containing the evaluation, null if unable to evaluate
    * @see Result
    */
  Result reduce(TokenType op, String attribute, String value);
}
