package lbl.akenti.parsers;

/**
  * Subclasses of this class only get to implement the
  * simple relational expressions.
  *
  * @author Abdelilah Essiari
  */
public class ExpressionEvaluator implements TokenType { 
  
  /**
    */
  public ExpressionEvaluator() {
  }

  /**
    * Returns the result of the evaluation of the expression <code>attribute op value</code>.
    *
    * @see TokenType
    */
  public boolean evaluate(int op, String attribute, String value) {
    throw new UnsupportedOperationException();
  }
 
  /**
    * Returns the result of the evaluation of the expression <code>attribute op value</code>.
    * Evaluation can result in yet another expression. 
    *
    * @return Result containing an expression or Result.NULL, Result.TRUE, Result.FALSE
    * @see Result
    */
  public Result reduce(int op, String attribute, String value) {
    return Result.NULL;
  }

  /**
    */
  public boolean evaluate(Expression expr) {
    return evaluate(expr.getRoot());
  }

  /**
    */
  public Result reduce(Expression expr) {
    Result ret = reduce(expr.getRoot());
    Object value = ret.getValue();
   
    if (value instanceof TreeNode) {
      return new Result(new Expression((TreeNode) value));
    } 

    return ret;
  }

  /**
    */
  private boolean evaluate(TreeNode n) {
    switch(n.getTokenType()) {
    case AND:
      return evaluate(n.getLeftChild())
             &&
             evaluate(n.getRightChild());
    case OR:
      return evaluate(n.getLeftChild())
             ||
             evaluate(n.getRightChild());
    case NOT:
      return !evaluate(n.getLeftChild());
    case ENULL:
      return true;
    case CONST:
    case INVALID:
      throw new RuntimeException();
    default:
      return evaluate(
        n.getTokenType(),
        n.getLeftChild().getData(),
        n.getRightChild().getData()
      );
    }
  }

  /**
    */
  Result
  reduce(TreeNode n) {
    switch(n.getTokenType()) {
      case AND:
        Result ret1 = reduce(n.getLeftChild());

        if (ret1.isFalse()) {
          return ret1;
        } else if (ret1.isTrue()) {
          return reduce(n.getRightChild());
        }
 
        Result ret2 = reduce(n.getRightChild());
 
        if (ret2.isFalse()) {
          return ret2;
        } else if (ret2.isTrue()) {
          return ret1;
        } 

        TreeNode n1 = (TreeNode) ret1.getValue();
        TreeNode n2 = (TreeNode) ret2.getValue();

        return new Result(new TreeNode(AND, AND_STRING, n1, n2));
      case OR:
        ret1 = reduce(n.getLeftChild());

        if (ret1.isTrue()) {
          return ret1;
        } else if (ret1.isFalse()) {
          return reduce(n.getRightChild());
        }

        ret2 = reduce(n.getRightChild());

        if (ret2.isTrue()) {
          return ret2;
        } else if (ret2.isFalse()) {
          return ret1;
        }

        n1 = (TreeNode) ret1.getValue();
        n2 = (TreeNode) ret2.getValue();
        return new Result(new TreeNode(OR, OR_STRING, n1, n2));
      case NOT:
        ret1 = reduce(n.getLeftChild());

        if (ret1.isTrue()) { 
          return Result.FALSE;
        } if (ret1.isFalse()) {
          return Result.TRUE;
        }

        n1 = (TreeNode) ret1.getValue();
        return new Result(new TreeNode(NOT, NOT_STRING, n1, null));
      case ENULL:
        return Result.TRUE;
      case CONST:
      case INVALID:
        throw new RuntimeException();
      default:
        Result ret = reduce(n.getTokenType(), 
                     n.getLeftChild().getData(),
                     n.getRightChild().getData());
        if (ret.isNull()) {
          return new Result(n);
        } else if (ret.isFalse() || ret.isTrue()) {
          return ret;
        }
         
        Expression expr = (Expression) ret.getValue();

        return new Result(expr.getRoot());
    }
  } 
}
