package lbl.akenti.policyEngine.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import lbl.akenti.certs.Certificate;
import lbl.akenti.policyEngine.CertificateHolder;
import lbl.akenti.policyEngine.PolicyContext;
import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.util.UTCTime;
import lbl.akenti.policyEngine.CertificateCache;

/**
  * No external caching is done for the moment. :-)
  * Life is beautiful.
  * 
  * @see lbl.akenti.policyEngine.CertificateHolder
  * @author Abdlelilah Essiari
  */
final class AkentiCertificateCache implements CertificateCache {
  private Map map = new HashMap();
  private UTCTime startCacheTime;
  private UTCTime endCacheTime;

  public AkentiCertificateCache(Runtime runtime) {
    PolicyContext ctx = runtime.getPolicyContext();
    long cacheInterval = ctx.getCacheInterval();
    startCacheTime = new UTCTime();
    endCacheTime = new UTCTime(cacheInterval);
  }

  public void addCertificateHolder(String key, CertificateHolder h) {
  }

  /**
    */
  public CertificateHolder addCertificate(String key, Certificate cert) {
    ArrayList holders = (ArrayList) map.get(key);

    if (holders  != null) {
      int idx = holders.indexOf(cert);

      if (idx != -1) {
        AkentiCertificateHolder holder = (AkentiCertificateHolder) holders.get(idx);

        holder.setFresh(false);
        return holder;
      }
    } else { 
      holders = new ArrayList();
      map.put(key, holders);
    }

    AkentiCertificateHolder holder = new AkentiCertificateHolder(cert);

    holders.add(holder);
    return holder;
  }

  /**
    */
  public CertificateHolder[] getCertificateHolders(String key) {
    ArrayList holders = (ArrayList) map.get(key);

    if (holders == null) {
      // GO TO EXTERNAL CACHE
      return new CertificateHolder[0]; 
    }

    return (CertificateHolder[]) holders.toArray(new CertificateHolder[0]);
  }

  /**
    */
  public boolean updateExternalCache() {
    return false;
  }
}
