package lbl.akenti.policyEngine.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.policyEngine.ResourcePolicy;

/**
  * Groups the <code>PolicyCertificate</code> and the <code>UseConditionCertificate</code>s
  * that apply to the same resource.<BR>
  *
  * @author Abdelilah Essiari.
  * @see lbl.akenti.PolicyCertificate
  * @see lbl.akenti.UseCondCertificate
  */

public final class AkentiResourcePolicy implements ResourcePolicy {

  private boolean isLeaf;
  
  /**
    * Policy certificate.
    */
  private PolicyCertificate policyCert;

  /**
    * UseCondition certificates.
    */
  private List ucCerts = new ArrayList();

  public void setLeaf(boolean isLeaf) {
    this.isLeaf = isLeaf;
  }
  
  public boolean isLeaf() {
    return isLeaf;
  }
    
  /**
    */
  public AkentiResourcePolicy(PolicyCertificate policyCert) {
    this.policyCert = policyCert;
  }

  /**
    */
  public String getResource() {
    return getPolicyCertificate().getResource();
  }

  /**
    */
  public PolicyCertificate getPolicyCertificate()  {
    return policyCert;
  }

  /**
    * Adds a <code>UseCondCertificate</code> to this <code>ResourcePolicy</code>
    */
  public void addUseCondCertificate(UseCondCertificate ucCert) {
    ucCerts.add(ucCert);
  }

  public UseCondCertificate[] toUseCondCertificates() {
    return (UseCondCertificate[]) ucCerts.toArray(new UseCondCertificate[0]);
  }

  /**
    */
  Iterator getUseCondCertificates()  {
    return ucCerts.iterator();
  }

  /**
    * Returns critical UCs.
    */
  List getCriticalUseCondCertificates()  {
    Iterator iter = getUseCondCertificates();
    List ret = new ArrayList();

    while (iter.hasNext()) {
      UseCondCertificate tempCert = (UseCondCertificate) iter.next();

      if (tempCert.isCritical()) {
        ret.add(tempCert);
      }
    }
    
    return ret;
  }

  /**
    * Returns non critical UCs.
    */
  List getNonCriticalUseCondCertificates()  {
    Iterator iter = getUseCondCertificates();
    List ret = new ArrayList();

    while (iter.hasNext()) {
      UseCondCertificate tempCert = (UseCondCertificate) iter.next();

      if (!tempCert.isCritical()) {
        ret.add(tempCert);
      }
    }

    return ret;
  }
}
