package lbl.akenti.policyEngine.impl;

import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.PublicKey;
import java.security.SignatureException;
import java.security.cert.X509Certificate;
import java.text.ParseException;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.util.UTCTime;
import org.w3c.dom.Element;

/**
  * Adapts X509 certificates to an AkentiCertificate.
  */
public class AkentiX509Certificate extends Certificate {

  private X509Certificate cert;

  public AkentiX509Certificate() {
    super("X509");
  }

  public AkentiX509Certificate(X509Certificate cert) {
    super("X509");
    this.cert = cert;
  }

  public X509Certificate getX509Certificate() {
    return cert;
  }

  /**
    **/
  public String getHashString() {
    return getSubject().to400String();  
  }

  /**
    * Returns the cersion number.
    */
  public int getVersion() {
    return cert.getVersion();
  }

  /**
    * Returns this <code>AkentiCertificate</code>'s unique id.
    */
  public String getUniqueID() {
    throw new UnsupportedOperationException();
  }

  /**
    */
  public DistinguishedName getSubject() {
    try {
      return new DistinguishedName(cert.getSubjectDN());
    } catch(ParseException ex) {
      throw new RuntimeException();
    }
  }

  /**
    * Returns this <code>AkentiCertificate</code>'s issuer.
    * Principal getIssuerDN();
    */
  public AkentiIssuer getIssuer() {
    try {
      return new AkentiIssuer(new AkentiPrincipal(new DistinguishedName(cert.getIssuerDN()), 
                                                                        new DistinguishedName()));
    } catch(ParseException ex) {
      throw new RuntimeException();
    }
  }

  /**
    * Returns the start validity date.
    */
  public UTCTime getNotBefore() {
    return new UTCTime(cert.getNotBefore());
  }

  /**
    * Returns the end validity date.
    */
  public UTCTime getNotAfter() {
    return new UTCTime(cert.getNotAfter());
  }

  /**
    * Returns the name of the signature algorithm.
    */
  public String getSignatureAlg() {
    return cert.getSigAlgName();
  }

  /**
    * Returns the name of the canonalization algorithm.
    */
  public String getCanonAlg() {
    return "ASN.1";
  }

  /**
    * Returns the signature.
    */
  public byte[] getSignature() {
    throw new UnsupportedOperationException();
  }

  /**
    * Returns the signable part of this certificate.
    *
    * @see writeObject(AkentiOutputStream)
    * @see doWriteObject(AkentiOutputStream)
    */
  public byte[] getSignable() throws IOException {
    throw new UnsupportedOperationException();
  }

  /**
    */
   public PublicKey getPublicKey() {
     return cert.getPublicKey();
   }

  /**
    */
  public boolean verify(PublicKey publicKey) 
    throws GeneralSecurityException, IOException {
    try {
      cert.verify(publicKey);
      return true;
    } catch(SignatureException ex) {
      return false;
    }
  }

  /**
    */
  protected Certificate newInstance()  {
    return new AkentiX509Certificate(); 
  }

  /**
    */
  public void writeObject(AkentiOutputStream drain) 
    throws IOException {
    throw new UnsupportedOperationException();
  }

  protected void readObject(AkentiInputStream source) 
    throws IOException , ParseException {
    throw new UnsupportedOperationException();
  }

  protected void fromXML(Element cert) throws ParseException {
    throw new UnsupportedOperationException();
  }
 
  public void toXML(Element cert) {
    throw new UnsupportedOperationException();
  }
}
