package lbl.akenti.policyManager.widgets;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import lbl.akenti.policyManager.base.GenAttribute;
import lbl.akenti.policyManager.base.GenValue;

/**
 * Allows the user to select an attribute/value pair.
 */
public class LBLAttrValPane
extends JOptionPane
implements ListSelectionListener, DocumentListener, ActionListener {
  
  private JList attributes;
  private JList values;
  private JTextField attrField;
  private JTextField valField;
  private JButton okButton;
  private DocumentListener docListener;
  
  /**
   */
  public LBLAttrValPane() {
    this(null);
  }
  
  /**
   */
  public LBLAttrValPane(String header) {
    this(null, true);
  }
  
  /**
   */
  public LBLAttrValPane(String header, boolean editable) {
    setLayout(new BorderLayout(10, 10));
    
    if (header != null) {
      add(LBLLabelFactory.createHtmlLabel(header), "North");
    }
    
    JLabel attrLabel = new JLabel("Attribute");
    
    attrLabel.setForeground(Color.red);
    attributes = new JList(new DefaultListModel());
    attributes.addListSelectionListener(this);
    attributes.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    
    JPanel temp1 = new JPanel(new BorderLayout());
    
    temp1.add(new JScrollPane(attributes), "Center");
    
    /* add the value list panel */
    JLabel valLabel = new JLabel("Value");
    
    valLabel.setForeground(Color.red);
    
    values = new JList(new DefaultListModel());
    values.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    values.addListSelectionListener(this);
    
    JPanel temp2  = new JPanel(new BorderLayout());
    
    temp2.add(new JScrollPane(values), "Center");
    
    attrField = LBLTextFieldFactory.createTextField(this);
    attrField.setEditable(editable);
    valField = LBLTextFieldFactory.createTextField(this);
    valField.setEditable(editable);
    
    add(
    WrapUtil.wrapComponents(
    WrapUtil.wrapVertical(WrapUtil.wrapHorizontal(attrLabel, attrField), temp1),
    WrapUtil.wrapVertical(WrapUtil.wrapHorizontal(valLabel,  valField), temp2)
    ),
    "Center"
    );
    
    doEnable();
  }
  
  public JDialog createDialog(Component parentComponent, String title) {
    if (okButton == null) {
      okButton = new JButton("OK");
      okButton.addActionListener(this);
      
      JButton cancelButton = new JButton("CANCEL");
      
      cancelButton.addActionListener(this);
      
      JPanel tempPanel = new JPanel();
      
      tempPanel.add(WrapUtil.wrapComponents(okButton, cancelButton));
      add(tempPanel, "South");
      doEnable();
    }
    
    return super.createDialog(parentComponent, "AttrValPane");
  }
  
    /*
     * Adds an Attribute.
     */
  public void addAttribute(GenAttribute attribute) {
    DefaultListModel model = (DefaultListModel) attributes.getModel();
    
    model.addElement(attribute);
  }
  
  /**
   */
  public void valueChanged(ListSelectionEvent e) {
    if (e.getSource() == attributes) {
      DefaultListModel model = (DefaultListModel) values.getModel();
      
      model.clear();
      attrField.setText("");
      valField.setText("");
      
      GenAttribute attribute = (GenAttribute) attributes.getSelectedValue();
      
      if (attribute != null) {
        attrField.setText(attribute.toString());
        
        Object[] values = attribute.getValues();
        
        for (int i = 0; i < values.length; ++i) {
          model.addElement(values[i]);
        }
      }
    } else {
      valField.setText("");
      GenAttribute attribute = (GenAttribute) attributes.getSelectedValue();
      
      if (attribute != null) {
        attrField.setText(attribute.toString());
        
        Object val = values.getSelectedValue();
        
        if (val != null) {
          valField.setText(val.toString());
        }
      }
    }
  }
  
  /**
   */
  public void setDocumentListener(DocumentListener docListener) {
    this.docListener = docListener;
  }
  
  /**
   */
  public void changedUpdate(DocumentEvent e)  {
    if (docListener != null) {
      docListener.changedUpdate(e);
    } else {
      doEnable();
    }
  }
  
  /**
   */
  public void insertUpdate(DocumentEvent e)  {
    if (docListener != null) {
      docListener.changedUpdate(e);
    } else {
      doEnable();
    }
  }
  
  /**
   */
  public void removeUpdate(DocumentEvent e) {
    if (docListener != null) {
      docListener.changedUpdate(e);
    } else {
      doEnable();
    }
  }
  
  /**
   */
  public void doEnable() {
    if (getAttributeName().equals("")) {
      doEnable(false);
    } else if (getValueName().equals("")) {
      doEnable(false);
    } else {
      doEnable(true);
    }
  }
  
  /**
   *
   */
  private void doEnable(boolean enable) {
    if (okButton != null) {
      if (enable) {
        LBLButtonPanel.enableBlueButton(okButton);
      } else {
        LBLButtonPanel.disableBlueButton(okButton);
      }
    }
  }
  
  /**
   */
  public void actionPerformed(ActionEvent event) {
    String cmd = event.getActionCommand();
    
    if (cmd.equals("OK")) {
      setValue(getGenAttribute());
    } else {
      setValue(GenConsts.AK_CANCEL_VALUE);
    }
  }
  
  /**
   */
  public GenValue getGenValue() {
    String valName = getValueName();
    GenValue value = new GenValue(valName);
    GenValue temp = (GenValue) values.getSelectedValue();
    
    if (temp != null && temp.hasValue(valName)) {
      value.addAkentiPrincipals(temp.getAkentiPrincipals());
      value.addDirs(temp.getDirs());
      value.addArgs(temp.getArgs());
    }
    
    return value;
  }
  
  /**
   */
  public GenAttribute getGenAttribute() {
    GenAttribute attribute = new GenAttribute("AKENTI", getAttributeName());
    
    attribute.addValue(getGenValue());
    return attribute;
  }
  
  /**
   */
  public String getAttributeName() {
    return attrField.getText().trim();
  }
  
  /**
   */
  public String getValueName() {
    return valField.getText().trim();
  }
  
  /**
   */
  public void clear() {
    DefaultListModel model = (DefaultListModel) attributes.getModel();
    
    model.clear();
    
    model = (DefaultListModel) values.getModel();
    model.clear();
    attrField.setText("");
    valField.setText("");
    setValue(null);
  }
  
}