/*
 * LBLBuildDirPane.java
 *
 * Created on August 4, 2004, 2:51 PM
 */

package lbl.akenti.policyManager.widgets;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import javax.naming.NamingException;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.*;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.policyManager.base.GenCa;
import lbl.akenti.util.LDAPSearchUtil;

/**
 * This class is a version of the build pane that has modified http and file browsing.
 * @author Tommaso Sciortino
 */
public class LBLBuildDirPane extends JOptionPane
implements ItemListener, ActionListener , DocumentListener {
  
  public static final String[] PROTOCOLS = { "ldap", "http", "file" };
  
  private final static String BUILD_DIR_PANEL_MESSAGE =
  "<html><font size=+1 color=blue>Build Directory" +
  "<li><font color=blue>Choose a protocol and provide " +
  "the appropriate components of a url" +
  "<li><font color=blue>hostname, port, file or search base if ldap....";
  
  protected final static String HELP_MESSAGE =
  "When protocol is \"file\" Host and Port are not applicable.";
  
  private JComboBox mComboBox = new JComboBox();
  
  protected JLabel baseLabel = new JLabel("");
  protected JTextField hostField;
  protected JTextField portField;
  protected JTextField fileField;
  
  private JButton helpButton;
  private JButton okButton;
  private JButton hintButton;
  protected JButton browse;
  private boolean dirSelect = false;
  
  private GenCa[] cas;
  
  private boolean isHintButtonVisible = true;
  /**
   * Constructor
   */
  public LBLBuildDirPane() {
    this(null);
  }
  
  /**
   */
  public LBLBuildDirPane(String []prots) {
    this(null, prots);
  }
  
  /**
   */
  public LBLBuildDirPane(String msg, String []prots) {
    setLayout(new BorderLayout(5, 5));
    
    if (msg == null) {
      msg = BUILD_DIR_PANEL_MESSAGE;
    }
    
    if (msg.startsWith("<html>")) {
      add(LBLLabelFactory.createHtmlLabel(msg), "North");
    } else {
      add(new AKTxtCanvas(msg), "North");
    }
    
    hostField = LBLTextFieldFactory.createTextField(": /", this);
    portField = LBLTextFieldFactory.createNumberField(this);
    fileField = LBLTextFieldFactory.createTextField(":", this);
    
    browse = new JButton("Browse");
    browse.setActionCommand("browse");
    browse.addActionListener(this);
    
    mComboBox.addItemListener(this);
    
    if (prots == null || prots.length == 0) {
      prots = PROTOCOLS;
    }
    
    for (int i = 0; i < prots.length; i++) {
      mComboBox.addItem(prots[i]);
    }
    
    itemStateChanged(null);
    
    JLabel labels[] = {
      new JLabel("Protocol"),
      new JLabel("Host"),
      new JLabel("Port"),
      baseLabel
    };
    
    for (int i = 0; i < labels.length; ++i) {
      labels[i].setForeground(Color.black);
    }
    
    JComponent []components = new JComponent[labels.length];
    
    browse.setEnabled(getProtocol().equals("ldap"));
    
    components[0] = mComboBox;
    components[1] = hostField;
    components[2] = portField;
    components[3] = WrapUtil.wrapComponents(fileField,browse);
    add(WrapUtil.wrapComponents(labels, components), "Center");
    
    hintButton = new JButton("Use Hints");
    LBLButtonPanel.disableBlueButton(hintButton);
    if(isHintButtonVisible) {
      hintButton.setVisible(true);
    }
    hintButton.addActionListener(this);
    
    JPanel temp = new JPanel();
    
    temp.add(hintButton);
    add(temp, "East");
    
    JPanel buttonPanel = new JPanel();
    
    helpButton = new JButton("Help");
    helpButton.setActionCommand("help");
    helpButton.addActionListener(this);
    buttonPanel.add(helpButton);
    
    okButton = new JButton("OK");
    okButton.setActionCommand("OK");
    okButton.addActionListener(this);
    buttonPanel.add(okButton);
    
    JButton cancelButton = new JButton("CANCEL");
    
    cancelButton.addActionListener(this);
    buttonPanel.add(cancelButton);
    
    add(buttonPanel, "South");
    doEnable();
  }
  
  /**
   */
  public void setGenCas(GenCa []cas) {
    this.cas = cas;
    doEnable();
  }
  
  /**
   */
  public void setOkLabel(String label) {
    okButton.setText(label);
  }
  
  /**
   */
  public void changedUpdate(DocumentEvent e)  {
    doEnable();
  }
  
  /**
   */
  public void insertUpdate(DocumentEvent e)  {
    doEnable();
  }
  
  /**
   */
  public void removeUpdate(DocumentEvent e) {
    doEnable();
  }
  
  
  /**
   */
  private void doEnable() {
    if (cas != null && cas.length != 0) {
      LBLButtonPanel.enableBlueButton(hintButton);
    } else {
      LBLButtonPanel.disableBlueButton(hintButton);
    }
    if (getProtocol().equals("file")) {
      doEnable(getFile().length() != 0);
    } else {
      doEnable(getHost().length() != 0);
    }
    
  }
  
  /**
   */
  private void doEnable(boolean on) {
    if (on) {
      LBLButtonPanel.enableBlueButton(okButton);
    } else {
      LBLButtonPanel.disableBlueButton(okButton);
    }
  }
  
  
  /**
   */
  public void actionPerformed(ActionEvent evt) {
    String cmd = evt.getActionCommand();
    
    if (cmd.equals("help")) {
      showDialog("Info", HELP_MESSAGE);
    } else if (cmd.equals("OK")) {
      setValue(getDirectory());
    } else if (cmd.equals("Use Hints")) {
      useHints();
    } else if (cmd.equals("browse")) {
      
      try{
        if (getProtocol().equals("ldap")) {
          browseLDAP();
        } else if (getProtocol().equals("file") || getProtocol().equals("http")) {
          browseFileHttp();
        }
      } catch(Exception ex) {
        LBLDialogFactory.showInfoDialog(this,
        ex.getClass().getName(),
        ex.getMessage());
      }
      
    } else {
      setValue(GenConsts.AK_CANCEL_VALUE);
    }
  }
  
  public void setDirectoriesOnly(boolean dirSelect) {
    this.dirSelect = dirSelect;
  }
  
  private void browseFileHttp() throws MalformedURLException, IOException{
    if (getProtocol().equals("http") && getHost().equals("")) {
      showMessage("You must enter a host to browse.");
      return;
    }
    
    int port = 80;
    if (!getPort().equals("")) {
      port = new Integer(getPort()).intValue();
    }
    
    URL url = new URL(getProtocol(), getHost(), port, getFile());
    
    HttpFileChooser d = new HttpFileChooser(url);
    if (dirSelect) {
      d.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
    }
    
    if (d.showOpenDialog(this) != d.APPROVE_OPTION) {
      return;
    }
    
    URL choosenURL = d.getSelectedURL();
    fileField.setText(choosenURL.getPath());
    
  }
  
  private void browseLDAP() throws MalformedURLException, NamingException{
    
    if (getHost().equals("")) {
      showMessage("You must enter a host to browse.");
      return;
    }
    
    int port = 389;
    if (!getPort().equals("")) {
      port = new Integer(getPort()).intValue();
    }
    
    URL url = new URL(getProtocol(), getHost(), port, "");
    String[] bases = LDAPSearchUtil.loadBases(url);
    if (bases.length == 0) {
      return;
    }
    
    String base = (String) JOptionPane.showInputDialog(this,"Choose from the selections below.",
    "Selection",JOptionPane.INFORMATION_MESSAGE,null,bases,bases[0]);
    
    if (base != null) {
      fileField.setText(base);
    }
    
  }
  
  /**
   */
  public void itemStateChanged(ItemEvent evt) {
    clearTextFields();
    
    String prot = getProtocol();
    
    browse.setEnabled(true);
    hostField.setEditable(true);
    portField.setEditable(true);
    baseLabel.setText("Base");
    
    if (prot.equalsIgnoreCase("file")) {
      hostField.setEditable(false);
      portField.setEditable(false);
      baseLabel.setText("File");
    } else if (prot.equalsIgnoreCase("https")) {
      browse.setEnabled(false);
    }
    return;
  }
  
  
  /**
   */
  public void clear() {
    cas = null;
    clearTextFields();
    doEnable();
    setValue(null);
  }
  
  /**
   */
  protected void clearTextFields() {
    hostField.setText("");
    portField.setText("");
    fileField.setText("");
  }
  
  /**
   */
  protected String getDirectory() {
    StringBuffer sb = new StringBuffer(getProtocol());
    
    String temp = getHost();
    
    sb.append(':');
    
    if (!temp.equals("")) {
      sb.append('/');
      sb.append('/');
      sb.append(temp);
      
      temp = getPort();
      
      if (!temp.equals("")) {
        sb.append(':');
        sb.append(temp);
      }
    }
    
    sb.append(getFile());
    return sb.toString();
  }
  
  /**
   */
  protected String getProtocol() {
    return (String) mComboBox.getSelectedItem();
  }
  
  /**
   */
  protected String getHost() {
    return (String) hostField.getText();
  }
  
  /**
   */
  protected String getPort() {
    return (String)portField.getText();
  }
  
  /**
   */
  protected String getFile() {
    String file = fileField.getText();
    
    if (file.length() == 0) {
      return "";
    }
    
    if (file.charAt(0) == '/') {
      return (String) file;
    }
    
    return "/" + file;
  }
  
  /**
   */
  void useHints() {
    if (cas.length == 0) {
      showMessage("<html><font color=red>No Certificate Authority Infos");
      return;
    }
    
    GenCa ca = (GenCa) LBLDialogFactory.showListDialog(
    this,
    "Certificate Authorities",
    "<html><font color=blue> Select a Certificate Authority",
    cas,
    0
    );
    
    if (ca == null) {
      return;
    }
    
    Directory []dirs = ca.getDirs();
    
    if (dirs.length == 0) {
      showMessage("<html><font color=red> No Directories for selected CA");
      return;
    }
    
    Directory dir = (Directory) LBLDialogFactory.showListDialog(
    this,
    "Directories",
    "<html><font color=blue> Select a directory",
    dirs,
    0
    );
    
    if (dir == null) {
      return;
    }
    
    URL url;
    
    try {
      url = new URL(dir.toString());
    } catch(MalformedURLException ex) {
      throw new RuntimeException("hummmm");
    }
    
    mComboBox.setSelectedItem(url.getProtocol());
    hostField.setText(url.getHost());
    
    int port = url.getPort();
    
    if (port != -1) {
      portField.setText(String.valueOf(port));
    }
    
    String base = url.getFile();
    
    if (base.startsWith("/")) {
      base = base.substring(1);
    }
    
    fileField.setText(base);
  }
  
  protected void showMessage(String message) {
    LBLDialogFactory.createInfoDialog(
    this,
    "Error Message",
    message
    ).setVisible(true);
  }
  
  public void setHintButtonStatus(boolean status) {
    hintButton.setVisible(status);
  }
  
  protected void showDialog(String title, String info) {
    LBLDialogFactory.createInfoDialog(
    this,
    title,
    (info == null? "" : info)
    ).setVisible(true);
  }
  
}