package lbl.akenti.policyManager.widgets;

import java.awt.BorderLayout;
import java.awt.Color;
import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import lbl.akenti.certs.elements.AkentiPrincipal;

/**
 * Allows a user to choose the certificate issuer.
 *
 * This sounds funny but the user and issuer are
 * one and the same and that is why we have two flavors:
 *
 * Optional: when we dont really know the issuers.
 * Required: when we know exactly who the issuers can be.
 */
public class LBLIssuerPanel extends LBLPanel implements ListSelectionListener {
  private JList list;
  
  private boolean optional;
  
  /**
   * Constructor.
   */
  public LBLIssuerPanel(LBLTopFrame parent, String header) {
    super(parent, header);
    
    
    JPanel temp = new JPanel(new BorderLayout(5, 5));
    JLabel label = new JLabel("Issuer, Issuer's Certificate Authority");
    
    label.setForeground(Color.red);
    temp.add(label, "North");
    
    list = new JList(new DefaultListModel());
    list.addListSelectionListener(this);
    temp.add(new JScrollPane(list) , "Center");
    add(temp, "Center");
  }
  
  /**
   */
  public void setOptional(boolean optional) {
    this.optional = optional;
    doEnable(false);
  }
  
  /**
   */
  public void valueChanged(ListSelectionEvent e) {
    doEnable(list.getSelectedIndex() != -1);
  }
  
  /**
   * Called When Cancel is clicked
   */
  public void clear() {
    DefaultListModel model = (DefaultListModel) list.getModel();
    
    model.clear();
    doEnable(false);
  }
  
  /**
   */
  private void doEnable(boolean on) {
    if(optional || on) {
      enableBlueButton("Next");
    } else {
      disableBlueButton("Next");
    }
    
    
  }
  
  public void doEnable() {
    if (optional) {
      enableBlueButton("Next");
    } else {
      disableBlueButton("Next");
    }
  }
  
  /**
   */
  public void setIssuers(AkentiPrincipal []issuers) {
    clear();
    
    DefaultListModel model = (DefaultListModel) list.getModel();
    
    for (int i = 0; i < issuers.length; i++) {
      model.addElement(issuers[i]);
    }
  }
  
  public AkentiPrincipal getIssuer() {
    return (AkentiPrincipal) list.getSelectedValue();
  }
  
}