package lbl.akenti.policyManager.widgets;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.StringTokenizer;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

/**
 *  Allows the user to provide the resource and the base url.
 */
public class LBLResourcePanel extends LBLPanel implements ActionListener, DocumentListener {
  
  private final static String DIR_PANEL_TITLE = "Directory Builder";
  private JTextField resourceField;
  private JButton browse = new JButton("Browse");
  private LBLTopFrame topFrame;
  private XMLHelper xmlHelper;
  private LBLDirField dirField;
  private AkentiResourceChooser fileChooser;
  
  /**
   * Constructor
   * Sets up the frame with default host and and port
   *
   * @param parent this window's parent
   * @param header the title of this window
   */
  public LBLResourcePanel(LBLTopFrame parent, String header) {
    super(parent, header);
    
    topFrame = parent;
    
    JLabel[] labels = { new JLabel("Resource"), new JLabel("base URL") };
    
    labels[0].setForeground(Color.red);
    labels[1].setForeground(Color.red);
    
    resourceField = LBLTextFieldFactory.createTextField(": ", this);
    
    browse.setActionCommand("browse");
    browse.addActionListener(this);
    
    JComponent resourceComponent = WrapUtil.wrapHorizontal(resourceField,browse);
    
    dirField = new LBLDirField(null, new String[] { "http", "https", "file" }, this);
    dirField.addDocumentListener(this);
    
    // check to see if .akenti directory exists in user's home directory
    File akentiDirectory = new File(System.getProperty("user.home") + System.getProperty("file.separator") + ".akenti");
    if (akentiDirectory.exists() && !akentiDirectory.isDirectory()) {
      throw new RuntimeException(".akenti is a file and should be a directory");
    }
    if (!akentiDirectory.exists()) {
      akentiDirectory.mkdir();
    }
    // check for history file
    xmlHelper = new XMLHelper(System.getProperty("user.home") + System.getProperty("file.separator") + ".akenti"
    + System.getProperty("file.separator") + "akenti_URLHISTORY.xml");
    
    if (xmlHelper.exists()) {
      load();
    }
    add(wrapComponents(labels, new JComponent[] { resourceComponent, dirField}),"Center");
    
    doEnable();
  }
  
  /**
   */
  AkentiResourceChooser getFileChooser() throws MalformedURLException, IOException{
    if (getBaseURL() == null && !getBaseURL().equals("")) {
      throw new IOException("You must type a base before browsing.");
    }
    
    URL url = new URL(getBaseURL());
    
    if (fileChooser == null || fileChooser.getBaseURL() != url) {
      fileChooser = new AkentiResourceChooser(url);
    }
    
    return fileChooser;
  }
  
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    
    if (cmd.equals("browse")) {
      
      try {
        
        URL url = new URL(getBaseURL());
        if (url.getProtocol().equals("https")) {
          if (!topFrame.initSSLInfo()) {
            return;
          }
        }
        
        fileChooser = getFileChooser();
        fileChooser.setCurrentDirectory(getResource());
        if (fileChooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION) {
          return;
        }
        
        String resName = fileChooser.getSelectedResourceName();
        
        resourceField.setText(resName);
        
      } catch(MalformedURLException ex) {
        LBLDialogFactory.showInfoDialog(this,
        ex.getClass().getName(),
        "You must type a base before browsing");
      } catch(IOException ex) {
        LBLDialogFactory.showInfoDialog(this,
        ex.getClass().getName(),
        ex.getMessage());
      }
      
    } else if (cmd.equals("create")) {
      LBLBuildDirPane optionPane = dirField.getDirOptionPane();
      optionPane.setDirectoriesOnly(true);
      
      optionPane.clear();
      optionPane.createDialog(this, DIR_PANEL_TITLE).setVisible(true);
      
      Object test = optionPane.getValue();
      if (test != null && !test.equals(GenConsts.AK_CANCEL_VALUE)) {
        String dir = (String) test;
        dirField.getComboBox().removeAllItems();
        dirField.getComboBox().addItem(dir);
        if (xmlHelper.exists()) {
          load();
        }
        optionPane.setValue(GenConsts.AK_CANCEL_VALUE);
      }
    } else {
      throw new RuntimeException(cmd + " not supported");
    }
    
  }
  
  /**
   */
  public void changedUpdate(DocumentEvent e)  {
    doEnable();
  }
  
  /**
   */
  public void insertUpdate(DocumentEvent e)  {
    doEnable();
  }
  
  /**
   */
  public void removeUpdate(DocumentEvent e) {
    doEnable();
  }
  
  private void doEnable() {
    
    if(getBaseURL() == null || getBaseURL().equals("")) {
      disableBlueButton("Next");
      disableBlueButton(browse);
    } else {
      enableBlueButton(browse);
      if (getResource() == null || getResource().equals("")) {
        disableBlueButton("Next");
      } else {
        enableBlueButton("Next");
      }
    }
    
  }
  
  /**
   */
  public void clear() {
    resourceField.setText("");
  }
  
  
  /**
   * Resource cannot begin with a slash and cannot end with a slash
   * It also cannot contina blank spaces.
   */
  public String getResource() {
    StringTokenizer st = new StringTokenizer(resourceField.getText(),"/");
    
    if (!st.hasMoreTokens()) {
      return "";
    }
    
    StringBuffer sb = new StringBuffer(st.nextToken());
    
    while (st.hasMoreTokens()) {
      sb.append('/');
      sb.append(st.nextToken());
    }
    
    String resource = sb.toString();
    
    return resource;
  }
  
  /**
   */
  public String getBaseURL() {
    String url = dirField.getDir();
    if (url != null) {
      if (url.equals("")) {
        return "";
      } else if (!url.endsWith("/")) {
        url += "/";
      }
    }
    return url;
  }
  
  public void load() {
    ArrayList items = (ArrayList) xmlHelper.decode();    
    for (int x = 0; x <= items.size() - 1; x++) {
      dirField.addItem(items.get(x));
    }
    
  }
  public void save() {
    if (xmlHelper.exists()) {
      ArrayList items = (ArrayList) xmlHelper.decode();
      if (!items.contains(dirField.getDir())) {
        // add the last entry USED to the beginning of the list
        items.add(0, dirField.getDir());
        xmlHelper.encode(items);
      } else {
         // add the last entry USED to the beginning of the list
         items.remove(items.indexOf(dirField.getDir()));
         items.add(0, dirField.getDir());
         xmlHelper.encode(items);
      }
      
    }
    else {
      createNewURLHistoryFile(true);
    }
  }
  
  public void createNewURLHistoryFile(boolean saveOne) {
    ArrayList toBeEncoded = new ArrayList();
    if (saveOne && dirField != null) {
      toBeEncoded.add(dirField.getDir());
    }
    xmlHelper.encode(toBeEncoded);
  }
  
}