package lbl.akenti.policyManager.widgets;

import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;

/**
 * Allows the <code>Policy Certificate Issuer</code> Issuer to set the certificate's
 * <code>expiration time</code>.
 *
 * @see javax.swing.JPanel
 */
public class LBLTimePanel extends LBLPanel {
  
  public static final String[] DURATION = {
    "1 Hour", "1 Day", "1 Week", "1 Month", "1 Year", "2 Years"
  };
  
  private JComboBox mComboBox = new JComboBox();
  
  /**
   * Constructor
   *
   * @param parent the parent of this object
   * @param header the title of this object
   */
  public LBLTimePanel(LBLTopFrame parent, String header) {
    super(parent, header);
    
    for (int i=0; i<DURATION.length; i++) {
      mComboBox.addItem(DURATION[i]);
    }
    
    JLabel labels[] = { new JLabel("Certificate Duration") };
    JComponent fields[] = new JComponent[labels.length];
    
    fields[0] = mComboBox;
    
    add(
    wrapComponents(labels , fields),
    "Center"
    );
  }
  
  /**
   */
  public String getLifeTimeAsString() {
    return (String) mComboBox.getSelectedItem();
  }
  
  /**
   */
  public long getLifeTime() {
    String time = getLifeTimeAsString();
    
    if (time.equals(DURATION[0])) {  // one hour
      return (long)1000*60*60;
    } else if (time.equals(DURATION[1])) {  // one day
      return (long)1000*60*60*24;
    } else if (time.equals(DURATION[2])) {  // one week
      return (long)1000*60*60*24*7;
    } else if (time.equals(DURATION[3])) {  // one month
      return (long)1000*60*60*24*30;
    } else if (time.equals(DURATION[4])) {  //one year
      return (long)1000*60*60*24*365;
    } else {  //two years
      return (long)1000*60*60*24*365*2;
    }
  }
  
  /**
   */
  public void clear() {
    mComboBox.setSelectedIndex(0);
  }
}