package lbl.akenti.policyManager.widgets;

import com.sun.net.ssl.HostnameVerifier;
import com.sun.net.ssl.HttpsURLConnection;
import com.sun.net.ssl.KeyManagerFactory;
import com.sun.net.ssl.SSLContext;
import com.sun.net.ssl.TrustManagerFactory;
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.security.cert.X509Certificate;
import java.util.BitSet;
import javax.net.ssl.SSLSocketFactory;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import lbl.akenti.policyManager.base.CredentialInfo;
import lbl.akenti.util.*;

/**
 * A border layout is used.
 *
 * The north is used for details about the frame.
 * The south is used for the navigation buttons
 * and the help button.
 */
public class LBLTopFrame extends JInternalFrame implements AkentiConstants {
  public static final int FIRST = 0;
  public static final int MIDDLE= 1;
  public static final int LAST =  2;
  public static final int VIEWER_LAST = 3;
  
  private LBLButtonPanel buttonPanel;
  private AbstractController controller;
  
  private static LBLTopFrame top;
  
  private boolean init = false;              // ssl initialization
  
  // Card Layout manager (necessary for transversing panels)
  private CardLayout cardManager;
  
  private JPanel cardManagerPanel;
  
  private int numberOfPanels;
  
  protected BitSet bits;       // one bit for each frame, if a bit is 0
  // loadInputsIntoPanel is called
  // The first bit is unused since we always show
  // the first frame.
  
  private static int openFrameCount = 0;
  private static final int XOFFSET = 30, YOFFSET = 30;
  private static final int RESET_WINDOW_LOCATION = 150;
  
  public static LBLTopFrame getInstance(String generatorName) {
    top = new LBLTopFrame(generatorName);
    return top;
  }
  
  /**
   * Constructor that actually sets up the frame.
   * All panels are the same except the first and last
   * one which have slightly different button configurations
   * at the bottom.
   */
  public LBLTopFrame(String generatorName) {
    
    super(generatorName + " " + (++openFrameCount),
    true, //resizable
    true, //closable
    true, //maximizable
    true);//iconifiable
    
    //Set the internal frame's location.
    setLocation(((XOFFSET * openFrameCount) % RESET_WINDOW_LOCATION), ((YOFFSET * openFrameCount) % RESET_WINDOW_LOCATION));
    
    JPanel contentPane = (JPanel) this.getContentPane();
    
    contentPane.setLayout(new BorderLayout(5, 5));
    contentPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
    
    addButtonPanel(FIRST);
    
    // set up the cardManger panel
    cardManagerPanel = new JPanel();
    cardManager = new CardLayout();
    cardManagerPanel.setLayout(cardManager);
    
    contentPane.add(cardManagerPanel, BorderLayout.CENTER);
    
  }
  
  /**
   * Purpose: To add a panel to this frame
   *
   * @param aPanel the panel to be added
   *        aLabel the name of the panel
   */
  public void addPanel(JPanel aPanel, String aLabel) {
    numberOfPanels++;
    cardManagerPanel.add(aPanel, aLabel);
  }
  
  public void initBitSet() {
    // at least one panel is required
    if (numberOfPanels >= 1) {
      bits = new BitSet(numberOfPanels);
    }
    else {
      throw new RuntimeException("Variable used to initialize BitSet isn't greater than 1.");
    }
  }
  
  /**
   */
  protected void addButtonPanel(int type) {
    String [] labels;
    
    switch(type) {
      case FIRST:
        labels = new String[] { "Help", "Close", "Next" };
        break;
      case MIDDLE:
        labels = new String[] { "Help", "Back", "Cancel", "Next" };
        break;
      case LAST:
        labels = new String[] { "Help", "Back", "Cancel", "Sign and Save"};
        break;
      case VIEWER_LAST:
        labels = new String[] { "Help", "Back", "Close"};
        break;
      default:
        throw new IllegalArgumentException();
    }
    
    LBLButtonPanel currentButtonPanel = buttonPanel;
    
    buttonPanel = new LBLButtonPanel();
    JButton button = null;
    for (int i = 0; i < labels.length; i++) {
      button = buttonPanel.addButton(labels[i]);
    }
    //button.setForeground(Color.white);
    //button.setBackground(Color.blue);
    buttonPanel.setActionListener(getActionListener());
    
    JPanel contentPane = (JPanel) this.getContentPane();
    
    if (currentButtonPanel != null) {
      contentPane.remove(currentButtonPanel);
    }
    
    contentPane.add(buttonPanel, "South");
  }
  
  /**
   * Resets all bits to 0.
   */
  public void clearBits() {
    bits.and(new BitSet());
  }
  
  /**
   * called when user presses cancel button, hides the current frame and
   * displays the first frame.
   */
  public void cancel() {
    getController().info = null;
    clearBits();
    addButtonPanel(FIRST);
    getController().clearLBLResourcePane();
    cardManager.first(cardManagerPanel);
    
  }
  
  protected ActionListener getActionListener() {
    return new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        
        try {
          String cmd = evt.getActionCommand();
          if (cmd.equals("Next")) {
            
            if (!bits.get(getPanelIndex() + 1)) {
              if (getController().loadInputsIntoPanel(getPanelIndex() + 1)) {
                bits.set(getPanelIndex() + 1);
              }
            }
            
            // check to see if button panel needs to be changed
            if (getPanelIndex() + 1 == numberOfPanels - 1) {
                if (numberOfPanels == 2) {
                    addButtonPanel(VIEWER_LAST);
                } else {
                    addButtonPanel(LAST);
                }
                
            } else {
              addButtonPanel(MIDDLE);
            }
            
            // show the next panel
            cardManager.next(cardManagerPanel);
            
            // check to enable "Next" button or not
            if(getPanelIndex() != numberOfPanels) {
              getController().checkEnableButton(getPanelIndex());
            }
            
          } else if (cmd.equals("Back")) {
            // check to see if button panel needs to be changed
            if (getPanelIndex() - 1 == 0) {
              addButtonPanel(FIRST);
            }
            else {
              addButtonPanel(MIDDLE);
            }
            
            // show the previous panel
            cardManager.previous(cardManagerPanel);
          } else if (cmd.equals("Cancel")) {
            cancel();
          } else if (cmd.equals("Sign and Save")) {
            getController().signAndSave();
          } else if (cmd.equals("Close")) {
            doDefaultCloseAction();
          } else if (cmd.equals("Help")) {
            showInfo(getController().getHelpMessage(getPanelIndex()));
          } else {
            throw new RuntimeException();
          }
        } catch(Exception ex) {
          showDialog("Error", ex.toString());
          ex.printStackTrace();
        }
      }
    };
  }
  
  protected int getPanelIndex() throws Exception {
    Component components[] = cardManagerPanel.getComponents();
    int index;
    for (index = 0; index  < components.length - 1; index++) {
      if (components[index].isShowing()) {
        break;
      }
    }
    return index;
  }
  
  /**
   */
  public void setController(AbstractController controller) {
    this.controller = controller;
  }
  
  /**
   */
  public AbstractController getController() {
    return controller;
  }
  
  /**
   */
  protected  void showError(String message) {
    showDialog("Error", message);
  }
  
  /**
   */
  protected  void showInfo(String message) {
    showDialog("Info", message);
  }
  
  /**
   */
  public void showDialog(String title, String info) {
    LBLDialogFactory.createInfoDialog(
    this,
    title,
    (info == null? "" : info)
    ).setVisible(true);
  }
  
  /**
   */
  public JButton getButton(String cmd) {
    return buttonPanel.getButton(cmd);
  }
  
  /**
   * Creates a panel with a label followed by a component on each line
   * labels[0]  components[0]
   * labels[1]  components[1] ...
   */
  protected JPanel wrapComponents(JLabel []labels, JComponent []components) {
    return WrapUtil.wrapComponents(labels, components);
  }
  
  /**
   * Wraps two components horizontally next to each other
   */
  protected JPanel wrapComponents(JComponent comp1, JComponent comp2) {
    return WrapUtil.wrapComponents(comp1, comp2);
  }
  
  /**
   * wraps two components vertically on top of each other
   */
  protected JPanel wrapVertical(JComponent comp1, JComponent comp2) {
    return WrapUtil.wrapVertical(comp1, comp2);
  }
  
  /**
   */
  protected void enableBlueButton(String cmd) {
    enableBlueButton(getButton(cmd));
  }
  
  /**
   */
  protected void disableBlueButton(String cmd) {
    disableBlueButton(getButton(cmd));
  }
  
  /**
   */
  protected void enableRedButton(String cmd) {
    enableRedButton(getButton(cmd));
  }
  
  /**
   */
  protected void disableRedButton(String cmd) {
    disableRedButton(getButton(cmd));
  }
  
  protected void enableBlueButton(JButton button) {
    LBLButtonPanel.enableBlueButton(button);
  }
  
  /**
   */
  protected void disableBlueButton(JButton button) {
    LBLButtonPanel.disableBlueButton(button);
  }
  
  /**
   */
  protected  void enableRedButton(JButton button) {
    LBLButtonPanel.enableRedButton(button);
  }
  
  /**
   */
  protected void disableRedButton(JButton button) {
    LBLButtonPanel.disableRedButton(button);
  }
  /**
   * Return whether or not initialized SSL info.
   */
  protected boolean initSSLInfo() {
    if (!init) {
      init = startSSLPane(this);
    }
    return init;
  }
  
  /**
   * Initializes SSL Parameters.
   */
  /**
   * Starts a pane asking the user for the pkcs12 file, passphrase, and
   * trusted CA file.
   */
  private boolean startSSLPane(LBLTopFrame parent) {
    // MOVE THIS HEADER
    String HEADER =
    "<html><font size=+1 color=blue> SSL Info" +
    "<li><font color=blue>Enter the name and password of the pkcs12 file.<i>" +
    "<li><font color=blue>Enter the name of the trusted CA(S) file."
    ;
    
    LBLSSLPane pane = new LBLSSLPane(HEADER);
    pane.setPkcs12FileName(System.getProperty("ClientPkcs12File", ClientPkcs12File));
    pane.setCAFileName(System.getProperty("CertificateAuthorityFile", CertificateAuthorityFile));
    
    // setFunctions
    JDialog dialog = pane.createDialog(parent, "SSL Info");
    
    do {
      dialog.show();
      
      if (pane.getValue().equals("CANCEL")) {
        break;
      }
      
      try {
        SSLInfo.initSSLInfo(
        pane.getPkcs12FileName(),
        pane.getPassword(),
        pane.getCAFileName()
        );
        
        CredentialInfo info = CredentialInfo.getInstance();
        
        info.readP12File(pane.getPkcs12FileName(), pane.getPassword());
        return true;
      } catch(IOException ex) {
        LBLDialogFactory.showInfoDialog(parent, ex.getMessage());
      } catch(GeneralSecurityException ex) {
        LBLDialogFactory.showInfoDialog(parent, ex.getMessage());
      }
    } while (true);
    
    // MEANS PERSON CANCEELED
    return false;
  }
  
  /**
   * Initializes SSL Parameters.
   */
  
  private void
  initSSLInfo(String p12FileName, String passwordString, String casFileName)
  throws IOException, GeneralSecurityException {
    char[] password = passwordString.toCharArray();
    KeyStore ks = KeyStore.getInstance("PKCS12");
    
    ks.load(new FileInputStream(p12FileName), password);
    
    X509CertReader xcr = new X509CertReader();
    X509Certificate[] caCerts = xcr.readX509Certs(casFileName);
    
    for (int i = 0; i < caCerts.length; ++i) {
      String alias = caCerts[i].getSubjectDN().toString();
      
      ks.setCertificateEntry(alias, caCerts[i]);
    }
    
    KeyManagerFactory kfac = KeyManagerFactory.getInstance("SunX509");
    
    kfac.init(ks, password);
    
    TrustManagerFactory tfac = TrustManagerFactory.getInstance("SunX509");
    
    tfac.init(ks);
    
    SSLContext ctx = SSLContext.getInstance("SSLv3");
    File f = new File(".rand");
    byte seed[] = null;
    
    try {
      if (f.exists()) {
        FileInputStream fis = new FileInputStream(f);
        byte temp[] = new byte[20];
        
        fis.read(temp);
        seed = temp;
      }
    } catch(IOException ex) {
      // DO NOTHING
    }
    
    SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
    
    if (seed == null) {
      seed = random.generateSeed(20);
      
      try {
        FileOutputStream fos = new FileOutputStream(f);
        fos.write(seed);
        fos.flush();
        fos.close();
       
      } catch(IOException ex) {
      }
    } else {
      random.setSeed(seed);
    }
    
    ctx.init(
    kfac.getKeyManagers(),
    tfac.getTrustManagers(),
    random
    );
    
    SSLSocketFactory sslFactory = ctx.getSocketFactory();
    HttpsURLConnection.setDefaultSSLSocketFactory(sslFactory);
    
    HttpsURLConnection.setDefaultHostnameVerifier(new HostnameVerifier() {
      public boolean verify(String urlHostname, String string) {
        return true;
      }
    });
  } 
}