package lbl.akenti.policyManager.widgets.pol;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.policyManager.widgets.GenConsts;
import lbl.akenti.policyManager.widgets.LBLPanel;
import lbl.akenti.policyManager.widgets.LBLPrincipalListPane;
import lbl.akenti.policyManager.widgets.LBLTopFrame;

/**
 *
 */
public class PolUCGroupsPanel
extends LBLPanel
implements ListDataListener, ListSelectionListener, ActionListener {
    
    private JList ucGroupList;
    
    private JButton viewGroupButton;
    private JButton remGroupButton;
    
    private LBLPrincipalListPane ucGroupPanel;
    
    /**
     * Constructs this frame.
     *
     * @param title title of the frame
     * @param instructions message that goes in the top portion of this frame
     */
    public PolUCGroupsPanel(LBLTopFrame parent, String header) {
        super(parent, header);
        
        JPanel temp = new JPanel(new BorderLayout(5, 5));
        JLabel label = new JLabel("Use Condition Issuer Groups");
        
        label.setForeground(Color.red);
        
        DefaultListModel model = new DefaultListModel();
        
        model.addListDataListener(this);
        ucGroupList = new JList(model);
        ucGroupList.setSelectionMode(2);
        ucGroupList.addListSelectionListener(this);
        temp.add(wrapVertical(label, new JScrollPane(ucGroupList)), "Center");
        
        JPanel buttonPanel = new JPanel();
        JButton tempButton = new JButton("Create Group");
        
        enableBlueButton(tempButton);
        buttonPanel.add(tempButton);
        tempButton.addActionListener(this);
        
        viewGroupButton = new JButton("View Group");
        viewGroupButton.addActionListener(this);
        buttonPanel.add(viewGroupButton);
        
        remGroupButton = new JButton("Remove Group");
        remGroupButton.addActionListener(this);
        buttonPanel.add(remGroupButton);
        temp.add(buttonPanel, "South");
        
        add(temp, "Center");
    }
    
    private static String UC_GROUP_PANEL_HEADER  =
    "<html><font size=+1 color=blue>UseConditionGroup Editor</font>\n\n" +
    "<li><font color=blue>Must provide at least one UseCondition issuer" +
    "<li><font color=blue>Must provide at least one UseCondition directory\n";
    ;
    
    /**
     */
    private LBLPrincipalListPane getUCGroupPanel() {
        if (ucGroupPanel == null) {
            ucGroupPanel = new LBLPrincipalListPane(
            UC_GROUP_PANEL_HEADER,
            "UseCond Issuers",
            "UseCond Directories"
            );
        }
        
        return ucGroupPanel;
    }
    
    
    /**
     */
    public void actionPerformed(ActionEvent event) {
        String cmd = event.getActionCommand();
        
        if (cmd.equals("Create Group")) {
            LBLPrincipalListPane ucGroupPanel = getUCGroupPanel();
            
            ucGroupPanel.setOKLabel("Add Group");
            ucGroupPanel.clear();
            ucGroupPanel.createDialog(this, "UCGroup Panel").setVisible(true);
            
            Object test = ucGroupPanel.getValue();
            
            if ((test != null) && !test.equals(GenConsts.AK_CANCEL_VALUE)){
                Object[] val = (Object[]) test;
                AkentiPrincipal[] principals = (AkentiPrincipal[]) val[0];
                Directory[] dirs = (Directory[]) val[1];
                
                addUCGroup(new UseCondIssuerGroup(principals, dirs));
            }
        } else if(cmd.equals("View Group")){
            LBLPrincipalListPane ucGroupPanel = getUCGroupPanel();
            int index = ucGroupList.getSelectedIndex();
            DefaultListModel model = (DefaultListModel) ucGroupList.getModel();
            UCGroup group = (UCGroup) model.getElementAt(index);
            UseCondIssuerGroup dgroup = group.getGroup();
            AkentiPrincipal []stakeholders = dgroup.getUCIssuers();
            Directory []ucDirs = dgroup.getDirectories();
            
            ucGroupPanel.clear();
            ucGroupPanel.addIssuers(stakeholders);
            ucGroupPanel.addDirs(ucDirs);
            ucGroupPanel.setOKLabel("Apply Changes");
            ucGroupPanel.createDialog(this, "UCGroupViewer").setVisible(true);
            
            Object test = ucGroupPanel.getValue();
            
            if ((test != null) && !test.equals(GenConsts.AK_CANCEL_VALUE)) {
                Object[] val = (Object[]) test;
                AkentiPrincipal[] principals = (AkentiPrincipal[]) val[0];
                Directory[] dirs = (Directory[]) val[1];
                
                group.setGroup(new UseCondIssuerGroup(principals, dirs));
                
                // Force the changes to appear graphically.
                model.removeElementAt(index);
                model.insertElementAt(group, index);
                ucGroupList.setSelectedIndex(index);
            }
        } else if(cmd.equals("Remove Group")){
            DefaultListModel model = (DefaultListModel) ucGroupList.getModel();
            int []indices = ucGroupList.getSelectedIndices();
            
            for (int i = 0; i < indices.length; ++i) {
                model.removeElementAt(indices[i]);
            }
        }
    }
    
    /**
     */
    public void clear() {
        DefaultListModel model = (DefaultListModel) ucGroupList.getModel();
        
        model.clear();
    }
    
    /**
     */
    public void valueChanged(ListSelectionEvent e) {
        doEnable();
    }
    
    /**
     */
    public void contentsChanged(ListDataEvent e) {
    }
    
    /**
     */
    public void intervalAdded(ListDataEvent e) {
    }
    
    /**
     */
    public void intervalRemoved(ListDataEvent e)  {
        doEnable();
    }
    
    /**
     */
    private void doEnable() {
        Object[] values = ucGroupList.getSelectedValues();
        
        if (values.length != 0) {
            enableRedButton(remGroupButton);
            
            if (values.length == 1) {
                enableBlueButton(viewGroupButton);
            } else {
                disableBlueButton(viewGroupButton);
            }
        } else {
            disableBlueButton(viewGroupButton);
            disableRedButton(remGroupButton);
        }
        
        DefaultListModel model = (DefaultListModel) ucGroupList.getModel();
        
        if (model.getSize() == 0) {
            disableBlueButton("Next");
        } else {
            enableBlueButton("Next");
        }
    }
    
    /**
     */
    public void setUCGroups(UseCondIssuerGroup[] groups) {
        clear();
        addUCGroups(groups);
    }
    
    /**
     */
    public int addUCGroups(UseCondIssuerGroup[] groups) {
        DefaultListModel model = (DefaultListModel) ucGroupList.getModel();
        int added = 0;
        int size = model.getSize();
        
        ucGroupList.clearSelection();
        
        for (int i = 0; i < groups.length; ++i) {
            int index = model.indexOf(groups[i]);
            
            if (index != -1) {
                ucGroupList.addSelectionInterval(index, index);
            } else {
                model.addElement(new UCGroup(groups[i]));
                ucGroupList.addSelectionInterval(size, size++);
                ++added;
            }
        }
        
        return added;
    }
    
    /**
     */
    public boolean addUCGroup(UseCondIssuerGroup group) {
        return addUCGroups(new UseCondIssuerGroup[] { group }) != 0;
    }
    
    /**
     */
    public UseCondIssuerGroup[] getUCGroups() {
        DefaultListModel model = (DefaultListModel) ucGroupList.getModel();
        UseCondIssuerGroup []dgroups = new UseCondIssuerGroup[model.getSize()];
        
        for (int i = 0; i < dgroups.length; ++i) {
            UCGroup wrapper = (UCGroup) model.getElementAt(i);
            
            dgroups[i] = wrapper.getGroup();
        }
        
        return dgroups;
    }
    
    private class UCGroup {
        private UseCondIssuerGroup group;
        
        UCGroup(UseCondIssuerGroup group) {
            this.group = group;
        }
        
        void setGroup(UseCondIssuerGroup group) {
            this.group = group;
        }
        
        UseCondIssuerGroup getGroup() {
            return group;
        }
        
        public String toString() {
            StringBuffer sb = new StringBuffer();
            
            sb.append("issuers");
            sb.append('=');
            sb.append(group.getUCIssuers().length);
            sb.append(',');
            sb.append("ucdirs");
            sb.append('=');
            sb.append(group.getDirectories().length);
            return sb.toString();
        }
    }
}

