package lbl.akenti.servers.AKMonitor.gui;

import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JOptionPane;
import javax.swing.JButton;
import java.awt.FlowLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Color;

/**
  * This panel keeps track of the connection state
  * by registring itself with the connection handler.
  *
  * Graphically, it manages two buttons connect and disconnect.
  * Only one of which is shown at any given time.
  *
  */
public class ConnectionPanel extends JPanel 
                             implements ConnectionListener, ActionListener {
  /* button used to connect */
  protected JButton connect;
  
  /* button used to disconnect */
  protected JButton disconnect;
  
  protected int port;
  protected String host;

  /*
   * Constructor.
   *
   * create the buttons, but add only the connect button
   * since we are initially disconnected.
   *
   * @param handler connection handler.
   **/
  public ConnectionPanel() {
    super(new FlowLayout(FlowLayout.LEFT, 0, 0));
    connect = new JButton("  connect ");
    connect.addActionListener(this);
    add(connect);
    disconnect = new JButton("disconnect");
    disconnect.addActionListener(this);
    disconnect.setBackground(Color.red);
    disconnect.setForeground(Color.white);
    LogHandler.getLogHandler().addConnectionListener(this);
  }
  
  public ConnectionPanel(String host, int port) {
    this();
    this.host = host;
    this.port = port;
  }

  public void setPort(int port) {
    this.port = port;
  }

  public void setHost(String host) {
    this.host = host;
  }

  /* a label to be put in the input dialog.  */
 
  protected JTextField field = new JTextField(20);
  protected Object [] message = new Object[] { "login:", field };
  protected Object [] list = new Object[] { "normal", "debug"};

  /*
   * show a dialog that accepts a user's login.
   * if user enters a nonempty string then call
   * handler.connect() 
   */
  protected void connect() {
    Object result = JOptionPane.showInputDialog(getTopLevelAncestor(),
                                                message,
                                                "Connect to Monitor Server",
                                                JOptionPane.OK_CANCEL_OPTION,
                                                null, 
                                                list,
                                                list[0]
                                               );

    
    if (result != null) {
      String uid = field.getText().trim();

      if (uid.equals("") == false) {
        LogHandler.getLogHandler().connect(host, port, uid);
      }
    } 
  }

  /*
   * call handler.connect() or handler.disconnect(). 
   *   
   **/
  public void actionPerformed(ActionEvent evt) {
    if (evt.getActionCommand().equals("  connect ")) {
      connect();
    } else {
      LogHandler.getLogHandler().disconnect();
      connectionStateChanged(new ConnectionEvent(this, ConnectionEvent.DISCONNECTED));
    } 
  }

  public boolean isConnected() {
    Object []comp = getComponents();

    if (comp.length == 0 || comp[0] == connect) {
      return false;
    }
    
    return true;
  }
 
  /*
   * show the right button depending on the state of 
   * the connection. 
   */
  public void connectionStateChanged(ConnectionEvent evt) {
    if (evt.isConnected()) {
      remove(connect);
      add(disconnect);
    } else {
      remove(disconnect);
      add(connect);
      
      if (!evt.getError().equals("")) {
        viewError(evt.getError());
      }
    } 
    
    validate();
    repaint();
  }
  
  /*
   * view error in an error dialog.
   **/
  protected void viewError(String error) {
    JOptionPane.showMessageDialog(null, error, "Connection Error", JOptionPane.ERROR_MESSAGE);
  }
}

