package lbl.akenti.servers.AKMonitor.gui;

/**
  * DisplayAcceccs.java
  * by Abdelilah Essiari
  *
  **/
import javax.swing.JApplet;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.DefaultListModel;
import javax.swing.border.Border;
import javax.swing.border.BevelBorder;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.WindowConstants;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.util.Hashtable;
import java.util.Enumeration;

public class DisplayAccess extends JApplet {
  MainWindow win;

  public void init() {
    String host = getCodeBase().getHost();
    int port = Integer.parseInt(getParameter("port"));
    
    win = new MainWindow("Access Viewer", host, port);
    win.pack();
    win.setSize(250, 400);
    win.setLocation(200, 200);
    win.show();
  }
  
  public void stop() {
    win.setVisible(false);
  }
  
  public void destroy() {
    win.dispose();
  }
}

class MainWindow extends JFrame implements StatusListener, ActionListener, ListSelectionListener {
  private ResourceViewer resourceViewer;
  private ResourceList list;
  private StatusPanel progressPanel;
  private JTabbedPane tabbedPanel;
  private ConnectionPanel connectionPanel;

  MainWindow(String title, String host, int port) {
    super(title);

    JPanel contentPane = (JPanel) getContentPane();
    
    contentPane.setLayout(new BorderLayout(10, 10));
    Border b = new EmptyBorder(7, 7, 7, 7); 
    
    contentPane.setBorder(b);

    progressPanel = new ProgressPanel("Progress Panel");
    contentPane.add("North", progressPanel);

    JPanel buttonPanel;
    JButton button;

    buttonPanel = new JPanel();
    connectionPanel = new ConnectionPanel(host, port);
    buttonPanel.add(connectionPanel);
    button = new JButton("clear");
    button.addActionListener(this);
    buttonPanel.add(button);

    JPanel mainPanel = new JPanel(new BorderLayout());

    mainPanel.add("Center", createResourceList());
    mainPanel.add("South", buttonPanel);

    buttonPanel = new JPanel();
    button = new JButton("previous");
    button.addActionListener(this);
    buttonPanel.add(button);
    button = new JButton("next");
    button.addActionListener(this);
    buttonPanel.add(button);
    resourceViewer = new ResourceViewer();
    resourceViewer.setBorder(b);

    JPanel resourcePanel = new JPanel(new BorderLayout());
    
    resourcePanel.add("Center", resourceViewer);
    resourcePanel.add("South", buttonPanel);
    
    tabbedPanel = new JTabbedPane();
    tabbedPanel.addTab("Main", mainPanel);
    tabbedPanel.addTab("Resource", resourcePanel);
    b = new BevelBorder(BevelBorder.RAISED); 
    tabbedPanel.setBorder(b);
    contentPane.add("Center", tabbedPanel);
    LogHandler.getLogHandler().addStatusListener(this);
    setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
  }
  
  protected JScrollPane createResourceList() {
    list = new ResourceList();

    JScrollPane scrollPanel = new JScrollPane(list);

    list.setBackground(scrollPanel.getBackground());
    list.setVisibleRowCount(30);
    list.addListSelectionListener(this);
    return scrollPanel;
  }

  public void actionPerformed(ActionEvent evt) {
    String cmd = evt.getActionCommand();

    if (cmd.equals("next")) {
      resourceViewer.next();
    } else if (cmd.equals("previous")) {
      resourceViewer.previous();
    } else if (cmd.equals("clear")) {
      if (connectionPanel.isConnected()) {
        list.clear();
      } else {
        resources.clear();
        list.clearAll();
      }

      resourceViewer.clear();
      progressPanel.clear();
    }
  }

  public void valueChanged(ListSelectionEvent e) {
    if (e.getValueIsAdjusting()) {
      return;
    }

    Resource resource = (Resource) list.getSelectedValue();
    
    if (resource.getStatus() == null && connectionPanel.isConnected()) {
      return;
    }

    resourceViewer.clear();
    resourceViewer.setResourceName(resource.getName());

    Enumeration events = resource.events();
    
    while (events.hasMoreElements()) {
      StatusEvent event = (StatusEvent) events.nextElement();

      resourceViewer.addStatusEvent(event);
    }
    
    tabbedPanel.setSelectedIndex(1);
  }

  private Hashtable resources = new Hashtable(20);

  public void addStatusEvent(int id, StatusEvent event) {
    if (id < 0) {
      Integer key = new Integer(-id);

      Resource resource = (Resource) resources.get(key);

      resources.remove(key);
      event = resource.lastStatusEvent();
      resource.setStatus(event.getStatus() == StatusEvent.SUCCESS);
      list.validate();
      list.repaint();
      return;
    } 

    Integer key = new Integer(id);
    Resource resource = (Resource) resources.get(key);

    if (resource != null) {
      resource.addStatusEvent(event);
    } else {
      resource = new Resource(event.getName());
      list.addResource(resource);
      resources.put(key, resource);
      resource.addStatusEvent(event);
    }
    
    progressPanel.addStatusCanvas(event);
  }
  
  public void dispose() {
    LogHandler.getLogHandler().disconnect();
    resourceViewer.dispose();
    super.dispose();
  }
}
    

