package lbl.akenti.servers.AKMonitor.gui;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.border.BevelBorder;
import java.awt.FlowLayout;
import java.awt.BorderLayout;
import java.awt.Color;
import java.util.NoSuchElementException;

/*
 * This <code>StatusPanel</code> contains a label 
 * and a panel to which * <code>StatusCanvas</code> 
 * canvases can be added.
 * 
 * The label can be used to give a title to the panel.
 */
public class StatusPanel extends JPanel {
  /*
   * panel to which canvases are added.
   */
  protected JPanel panel;

  /*
   * optional label to be view a title.
   */
  protected JLabel label;

  /*
   * Contructor.
   */
  public StatusPanel() { 
    statusPanelInit();
  }

  /*
   * initialize label and panel.
   */
  protected void statusPanelInit() {
    setLayout(new BorderLayout());
    setBorder(new BevelBorder(BevelBorder.LOWERED));
    panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 3, 3));
    panel.setBackground(Color.white);
    panel.setBorder(new BevelBorder(BevelBorder.LOWERED));
    label = new JLabel();
    label.setVerticalTextPosition(label.TOP);
    label.setHorizontalAlignment(label.LEFT);
    label.setHorizontalTextPosition(label.LEFT);
    label.setForeground(Color.black);
    add("North", label);
    add("Center", panel);
  }

  /*
   * set the text of the label.
   */
  public void setTitle(String title) {
    label.setText(title);
  }

  /*
   * create a canvas associated with the argument event;
   * and then add it to the panel.
   */
  public StatusCanvas addStatusCanvas(StatusEvent event) {
    StatusCanvas canvas = new StatusCanvas(event); 

    panel.add(canvas); 
    return canvas;   
  }
  
  /*
   * add a canvas to the panel.
   */
  public StatusCanvas addStatusCanvas(StatusCanvas canvas) {
    panel.add(canvas); 
    return canvas;   
  }

  /*
   * @return the canvases that were added to the panel.
   */
  public StatusCanvas [] canvases() {
    Object [] objects = panel.getComponents();
    StatusCanvas canvases[] = new StatusCanvas[objects.length];
    
    for (int i = 0; i < objects.length; i++) {
      canvases[i] = (StatusCanvas) objects[i];
    }

    return canvases; 
  }
   
  /*
   * @return the first canvas in the panel.
   */
  public StatusCanvas firstCanvas() throws NoSuchElementException {
    int count = panel.getComponentCount();
    
    if (count == 0) {
      throw new NoSuchElementException();
    }

    return (StatusCanvas) panel.getComponent(0);
  }

  /*
   * @return the last canvas in the panel.
   */
  public StatusCanvas lastCanvas() throws NoSuchElementException {
    int count = panel.getComponentCount();

    if (count == 0) {
      throw new NoSuchElementException();
    }
   
    return (StatusCanvas) panel.getComponent(count - 1);
  }

  /*
   * remove all canvases from panel. 
   */
  public void clear() {
    panel.removeAll();
  }
}
  
   
  
