package lbl.akenti.servers.AKMonitor.util;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedInputStream;

import java.util.StringTokenizer;


/**
  * AkentiEventParser.java
  * simple parser to parse an InputStream into AkentiEvent
  * @author Guillaume Farret
  * @version August 2001
  */
public class AkentiEventParser implements AkentiEventConstants {
  
  private BufferedReader reader;
  
  private int phase = 0;//phase of the currently parsed event

  public AkentiEventParser(InputStream input) {
    this.reader = new BufferedReader(new InputStreamReader(new BufferedInputStream(input)));
  }

  public AkentiEvent nextEvent(StringBuffer sb) throws IOException {
    AkentiEvent event;
    StringBuffer lineB = new StringBuffer();    
  
    while(true){
      lineB.append(reader.readLine());
      if(isNewLine(reader))break;
    };

    //stringBuffer used to store all incoming event to write to a file.
    sb.append(lineB.toString());
    sb.append('\n');

    event = new AkentiEvent();
    System.out.println(lineB.toString());
    if(lineB.toString().equals("null"))return null;
    //fill the event table
    fillEventTable(lineB.toString(),event);

    return finalizeEvent(event);
  }


/**
  * fill the has table of the given event with (key,value) pair parsed
  * from the given String
  * @param logline the given String to parse
  * @param event the event to fill
  */
  private void fillEventTable (String logLine, AkentiEvent event) {
    try {
      String key = "";
      String value = "";
      int start=0,end=0;
      
      while (end<logLine.length()) {
        
        end = logLine.indexOf('=',end);
        if (end == -1) {
          break;
        }
        
        key = logLine.substring(start,end++);

        while (logLine.charAt(end) == ' ') {end++;}//remove ' ' after the '='
        
        start = end;
        
        if (logLine.charAt(end++)=='"') {//if value begin with '"'
          
          end = logLine.indexOf('"',end);
          if (end < 0) {
            break;
          }
          
          value = logLine.substring(start+1,end);
          start = ++end;
        } else { // if value does not begin with '"'
          
          end = logLine.indexOf('=',end);
          if (end < 0) {//last value
            end = logLine.length();
          } else {//if not last value
            while (logLine.charAt(--end) == ' ') {}
            while (logLine.charAt(--end) != ' ') {}
          }
          
          if (start > end) {
            event.table.put(
              "BAD_FORMATED_LOG",
              logLine.substring(start,logLine.length())
            );
            break;
          }
          value = logLine.substring(start,end);
          start = end++;
        }
        //System.err.println("key: "+key.trim()+" value: "+value.trim());
        event.table.put(key.trim(), value.trim());
      }
    } catch (Exception e) {
      e.printStackTrace(System.err);
    }
  }
    

/**
  * function that return true if reader is going to read a new event
  *
  * @param reader
  */ 
  private boolean isNewLine(BufferedReader reader) throws IOException {
    char[] buffer = new char[5];
    reader.mark(5);
    int  EOF = reader.read(buffer,0,5);
    reader.reset();
    if (String.valueOf(buffer).equals("DATE=") || EOF == -1) {//a new log line begin with "DATE="
      return true;
    } else {
      return false;
    }
  }
  

/** NO MORE USED
  * function that gives the value of a key within a log line
  * @param key key to search the value  (ex: NL.EVNT, LVL ...)
  * @param line string to be searched in
  *
  * @return string value of the requested attribute,
  *         return "NO_VALUE" if the key was not found 
  */
  public static String valueOfAttribute(String key, String line) throws IllegalArgumentException {
    
    if (key == null || line == null) throw new IllegalArgumentException();
    
    int start = line.indexOf(key);

    if (start == -1) return "NO_VALUE";//key not found
    start += key.length();//point just after the key

    while (line.charAt(start) != '=') {//point just after the = sign, (skip all blankspace before)
      if (line.charAt(start) != ' ') return "NO_VALUE";
      start++;
    }

    do {start++;} while (line.charAt(start) == ' ');//skip all blank space after = sign
    int end=0;

    if (line.charAt(start) == '"') {//if begin with " sign then point to the end " sign
      start++;
      end = start;
      while (end < line.length() && line.charAt(end) != '"') {end++;}
    } else {//else point to the next blankspace
      end = line.indexOf(' ',start);
      if (end == -1 )end = line.length();
    }
    return line.substring(start,end);//return the string between the start and end pointer
  }
  
/**
  * function that set the status, message, and level value of an AkentiEvent
  *
  * @param event AkentiEvent to be finalized
  */ 
  private AkentiEvent finalizeEvent(AkentiEvent event) {
    String status = (String)event.table.get("STATUS");
    String msg = (String)event.table.get("MSG");
    String key = (String)event.table.get("NL.EVNT");
    String level = (String)event.table.get("LVL");
    
    if (key != null) {
      event.setKey(key);
    } else {
      event.setKey("NO_KEY");
    }

    if (level != null) {
      event.level = level;
    }

    if (msg != null) {
      event.message = msg;
    }
    
    if (status != null) {
      if (status.equals("BEGIN")) {
        event.status = PENDING;
      } else if (status.equals("SUCCESS")) {
        event.status = SUCCESS;
      } else if (status.equals("FAIL")) {
        if (event.key.equals("CHECK_ACCESS")) {
          event.status = DENIED;
        } else {
          event.status = FAILURE;
        }
      }
    }
    
    if (event.key.equals("UC_COLLECTION_PHASE") && event.status == PENDING) {
      event.phase = phase = 1;
    } else if (event.key.equals("AUTHORIZATION_PHASE") && event.status == PENDING) {
      event.phase = phase = 2;
    } else if (event.key.equals("CHECK_ACCESS") && event.status != PENDING) {
      event.phase = phase = 3;
    } else {
      event.phase = phase;
    }
      

    return event;
  }


/**
  * test main method
  */
  public static void main (String[] args) {
    try{
      AkentiEventParser t = new AkentiEventParser(System.in);
      AkentiEvent event;
      StringBuffer sb = new StringBuffer();

      while (true) {
        event = t.nextEvent(sb);
        if (event == null) break;

        System.out.println(event.toString());
      }

    }catch(Exception e){
      e.printStackTrace(System.out);
    } 
  }
} 

