package lbl.akenti.servers.CacheManager;

import java.io.PrintWriter;
import java.io.ByteArrayOutputStream;
import java.io.LineNumberReader;
import java.io.IOException;
import java.util.StringTokenizer;
import java.text.ParseException;

/**
  *
  */
public class CacheRequest {

  private String method;

  private String rootResourceName;

  private String certHash;

  private String dirName;

  private CacheCertificate cert;

  /**
    *
    */
  public CacheRequest() {
    this("NO_METHOD");
  }

  /**
    *
    */
  public CacheRequest(String method) {
    if (method == null || method.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.method = method;
    this.rootResourceName = "NO_RESOURCE";
    this.certHash = "NO_HASH";
    this.dirName = "";
  }
  
  /**
    *
    */
  public String getMethod() {
    return method;
  }

  /**
    *
    */
  public boolean hasMethod(String method) {
    return method.equalsIgnoreCase(this.method);
  }

  /**
    *
    */
  public void setRootResourceName(String rootResourceName) {
    if (rootResourceName == null || rootResourceName.length() == 0) {
      throw new IllegalArgumentException();
    }

    this.rootResourceName = rootResourceName; 
  }

  /**
    *
    */
  public String getRootResourceName() {
    return rootResourceName;
  }

  /**
    *
    */
  public void setCertHash(String certHash) {
    if (certHash == null || certHash.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    this.certHash = certHash; 
  }

  /**
    *
    */
  public String getCertHash() {
    return certHash;
  }

  /**
    *
    */
  public void setDirName(String dirName) {
    if (dirName == null) {
      throw new IllegalArgumentException();
    }

    this.dirName = dirName; 
  }

  /**
    *
    */
  public String getDirName() {
    return dirName;
  }

  /**
    *
    */
  public void setCertificate(CacheCertificate cert) {
    if (cert == null) {
      throw new IllegalArgumentException();
    }
    
    this.cert = cert;
  }

  /**
    *
    */
  public CacheCertificate getCertificate() {
    return cert;
  }

  /**
    *
    */
  public UtcTime getNotAfter() {
    return getCertificate().getNotAfter();
  }

  protected void writeHeaderInfo(PrintWriter pw) {
    pw.println("METHOD: " + method);
  }

  protected void writeResourceInfo(PrintWriter pw) {
    pw.println("ROOT_RESOURCE_NAME: " + rootResourceName);
    pw.println("CERT_HASH: " + certHash);
    
    if (dirName.length() != 0) {
      pw.println("DIRNAME: " + dirName);
    }
  }

  protected void writeCertInfo(PrintWriter pw) {
    if (cert != null) {
      String temp = cert.getCertificate();

      pw.println("CERT_EXP_TIME" + ": " + cert.getNotAfter());
      pw.println("CERT_LENGTH" + ": " + temp.length());
      pw.println(temp);
    }
  }

  public void writeExternal(PrintWriter pw) {
    writeHeaderInfo(pw);
    writeResourceInfo(pw);
    writeCertInfo(pw);
    pw.println();
    pw.flush();
  }

  protected void readHeaderInfo(LineNumberReader lnr) throws ParseException, IOException {
    String line = readLine(lnr);

    method = parse("METHOD", line, lnr.getLineNumber());
  }
    
  protected void readResourceInfo(LineNumberReader lnr) throws ParseException, IOException {
    String line = readLine(lnr);

    rootResourceName = parse("ROOT_RESOURCE_NAME", line, lnr.getLineNumber());
    line = readLine(lnr);
    certHash = parse("CERT_HASH", line, lnr.getLineNumber());
    lnr.mark(80);
    line = readLine(lnr);
    
    if (line.startsWith("DIRNAME")) {
      dirName = parse("DIRNAME", line, lnr.getLineNumber());
    } else {
      dirName = "";
      lnr.reset();
    }
  }

  protected void readCertInfo(LineNumberReader lnr) throws ParseException, IOException {
    String line = readLine(lnr);

    UtcTime notAfter = new UtcTime(parse("CERT_EXP_TIME", line, lnr.getLineNumber()));
    line = readLine(lnr);

    String temp = parse("CERT_LENGTH", line, lnr.getLineNumber());
       
    for (int i = 0; i < temp.length(); ++i) {
      if (!Character.isDigit(temp.charAt(i))) {
        throw new ParseException("Failed to read length of cert", lnr.getLineNumber());
      }
    }

    int len = new Integer(temp).intValue() + 1;
    char[] chars = new char[len];

    if (lnr.read(chars, 0, len) != len) {
      throw new ParseException("Failed to read cert", lnr.getLineNumber());
    }
    
    cert = new CacheCertificate(new String(chars, 0, len - 1), notAfter);
  }

  public void readExternal(LineNumberReader lnr) throws ParseException, IOException {
    readHeaderInfo(lnr);
    readResourceInfo(lnr);
    cert = null;
    lnr.mark(80);
    String line = readLine(lnr);

    if (line.length() != 0) {
      lnr.reset();
      readCertInfo(lnr);

      if (readLine(lnr).length() != 0) {
        throw new ParseException("Failed to read eline", lnr.getLineNumber());
      }
    }
  }

  public String toString() {
    ByteArrayOutputStream baos = new  ByteArrayOutputStream();
     
    writeExternal(new PrintWriter(baos));
    return baos.toString();
  }

  protected String parse(String keyword, String line, int lineNumber) throws ParseException {
    StringTokenizer tokenizer = new StringTokenizer(line, " =:");

    if (tokenizer.countTokens() != 2) {
      throw new ParseException("badly formed line", lineNumber);
    }

    if (!keyword.equalsIgnoreCase(tokenizer.nextToken())) {
      throw new ParseException("can't find keywod " + keyword, lineNumber);
    }

    return tokenizer.nextToken();
  }

  protected String readLine(LineNumberReader lnr) throws ParseException, IOException {
    String line = lnr.readLine();

    if (line == null) {
      throw new ParseException("did not expect eof after line ", lnr.getLineNumber());
    }
    
    return line;
  }
}
