package lbl.akenti.servers.ResDefServer;

import com.sun.net.ssl.KeyManagerFactory;
import com.sun.net.ssl.SSLContext;
import com.sun.net.ssl.TrustManagerFactory;
import com.sun.net.ssl.internal.ssl.Provider;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.io.OutputStream;
import java.net.Socket;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.security.Security;
import java.security.cert.X509Certificate;
import java.util.Properties;
import javax.net.SocketFactory;
import lbl.akenti.util.P8KeyStore;
import lbl.akenti.util.P8Reader;
import lbl.akenti.util.X509CertReader;

/**
  * This http client performs a simple GET request in secured mode to the server at
  * localhost at port number 8080.
  *
  */
public class Client {
  public static void main(String[]args) {
    try {
      Security.addProvider(new Provider());

      System.setProperty("java.protocol.handler.pkgs", "com.sun.net.ssl.internal.www.protocol");

      KeyStore ks = new P8KeyStore();

      ks.load(null, null);

      P8Reader p8Reader = new P8Reader();

      p8Reader.readPrivateKey("clientKey.pem");
      p8Reader.readX509Chain("clientCert.pem");

      ks.setKeyEntry(p8Reader.getSubjectDN().toString(),
                     p8Reader.getPrivateKey(),
                     null,
                     p8Reader.getX509CertificateChain());

      X509CertReader xcr = new X509CertReader();
      String casFileName = "cas.pem";
      X509Certificate[] caCerts = xcr.readX509Certs(casFileName);

      for (int i = 0; i < caCerts.length; ++i) {
        String alias = caCerts[i].getSubjectDN().toString();

        ks.setCertificateEntry(alias, caCerts[i]);
      }

      KeyManagerFactory kfac = KeyManagerFactory.getInstance("SunX509");

      kfac.init(ks, null);

      TrustManagerFactory tfac = TrustManagerFactory.getInstance("SunX509");

      tfac.init(ks);

      SSLContext ctx = SSLContext.getInstance("SSLv3");

      File f = new File(".rand");
      SecureRandom random;

      random = SecureRandom.getInstance("SHA1PRNG");
      byte seed[] = null;

      try {
        if (f.exists()) {
          FileInputStream fis = new FileInputStream(f);
          byte temp[] = new byte[20];

          fis.read(temp);
          seed = temp;
        }
      } catch(IOException ex) {
      }

      if (seed == null) {
        System.out.println("this will take some time...");
        seed = random.generateSeed(20);

        try {
          FileOutputStream fos = new FileOutputStream(f);
          fos.write(seed);
          fos.flush();
          fos.close();

          System.out.println("A .rand file is saved into this directory");

        } catch(IOException ex) {
        }
      } else {
        random.setSeed(seed);
      }

      ctx.init(
        kfac.getKeyManagers(),  
        tfac.getTrustManagers(),
        random                   
      );

      SocketFactory factory = ctx.getSocketFactory();
      int port = 8080;

      Socket socket = factory.createSocket("localhost", port);

      OutputStream os = socket.getOutputStream();
      String req = "GET ";
      final String CRLF = "\r\n";

      req += "/";

      req += " HTTP/1.0" + CRLF;
      req += "Content-Type: text/plain" + CRLF; 

      req += CRLF;

      os.write(req.getBytes());
      os.flush();
 
      InputStream is = socket.getInputStream();
      InputStreamReader r = new InputStreamReader(is);
      LineNumberReader r2 = new LineNumberReader(r);

      while (true) {
        String line = r2.readLine();

        if (line == null) {
          break;
        }

        System.out.println(line);
      }
    } catch(Exception ex) {
      ex.printStackTrace();
    }

  }
}
