package lbl.akenti.servers.ResDefServer;

import java.io.InputStream;

/** 
  * A HttpRequest object encapsulates information about a single client request.
  * 
  * @author Jiewei Lin
  */
public class HttpRequest {
  private InputStream in = null; 
  private String uri = "";
  private String method = "";
  private String contentType = "";
  private int contentLength = 0;

  /** 
    * Default constructor. 
    */
  public HttpRequest() {
  }

  /** 
    * Returns the requested URI. 
    *
    * @return user's requested URI.
    */
  public String getURI( ) {
    return uri;
  } 

  /** 
    * Sets URI.
    * 
    * @param uri requested URI.  
    * @exception IllegalArgumentException is thrown if parameter uri is null.
    */
  void setURI(String uri) {
    if (uri == null) {
      throw new IllegalArgumentException();
    }
    this.uri = uri;
  }
 
  /** 
    * Returns an InputStream object.
    * 
    * @return an InputStream object.
    */ 
  public InputStream getInputStream() {
    return in;
  }

  /** 
    * Sets InputStream.
    * 
    * @param in InputStream object to get client's data.
    * @exception IllegalArgumentException is thrown if parameter in is null. 
    */
  void setInputStream(InputStream in) {
    if (in == null) {
      throw new IllegalArgumentException();
    }
    this.in = in;
  }

  /** 
    * Returns requested method, usually GET, or POST.  It can be HEAD, PUT, or DELETE.
    * 
    * @return either GET, POST, HEAD, PUT, or DELETE.
    */ 
  public String getMethod() {
    return method;
  }

  /** 
    * Sets method.
    *
    * @param method requested method, usually GET, POST.  It can be Head, PUT, or DELETE.
    * @exception IllegalArgumentException is thrown if parameter method is null.
    */
  void setMethod(String method) {
    if (method == null) {
      throw new IllegalArgumentException();
    }
    this.method = method;
  }

  /** 
    * Returns request content type.
    *
    * @return request content type.
    */
  public String getContentType() {
    return contentType;
  }

  /** 
    * Sets content type.
    *
    * @param contentType content type.
    * @exception IllegalArgumentException is thrown if parameter contentType is null.
    */
  void setContentType(String contentType) {
    if (contentType == null) {
      throw new IllegalArgumentException();
    }
    this.contentType = contentType;
  }

  /** 
    * Returns request content length.
    *
    * @return request content length.
    */
  public int getContentLength() {
    return contentLength;
  } 

  /** 
    * Sets content length.
    *
    * @param contentLength content length. 
    * @exception IllegalArgumentException is thrown if parameter contentLength is less 
    *   than 0.
    */
  void setContentLength(int contentLength) {
    if (contentLength < 0) {
      throw new IllegalArgumentException();
    }
    this.contentLength = contentLength;
  }
}
