package lbl.akenti.servers.ResDefServer;

import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;

/** 
  * A HttpResponse object encapsulates information about a single client response.
  *
  * @author Jiewei Lin
  */
class HttpResponse {
  OutputStream out = null;
  private int statusCode = -1;     
  private String contentType = ""; 

  /** 
    * Default constructor. 
    */
  HttpResponse() {
  }

  /** 
    * Returns an OutputStream object for users to write their messages to the client.
    *
    * @return OutputStream for users to write to.
    */
  public OutputStream getOutputStream() {
    return out;      
  }

  /** 
    * Sets OutputStream object out.
    *
    * @param out for user to write to. 
    * @exception IllegalArgumentException is thrown if parameter out is null.
    */
  void setOutputStream(OutputStream out) {
    if (out == null) {
      throw new IllegalArgumentException();
    }
    this.out = out;
  } 

  /** 
    * Returns a PrintWriter object for users to write their messages to the client.
    *
    * @return PrintWriter for users to write to.
    */
  public PrintWriter getWriter() {
    return new PrintWriter(new OutputStreamWriter(out));    
  }
 
  /** 
    * Gets the response status code. 
    *
    * @return the response status code.
    */
  public int getStatusCode() {
    return statusCode;
  } 

  /** 
    * Sets response status code.
    *
    * @param statusCode response status code.
    * @exception IllegalArgumentException is thrown if parameter statusCode is less than 0.
    */
  void setStatusCode(int statusCode) {
    if (statusCode < 0) {
      throw new IllegalArgumentException();
    }
    this.statusCode = statusCode;
  }

  /** 
    * Returns content type.
    *
    * @return content type.
    */
  public String getContentType() {
    return contentType;
  }

  /** 
    * Sets content type.
    *
    * @param contentType content type.
    * @exception IllegalArgumentException is thrown if parameter contentType is null.
    */
  void setContentType(String contentType) {
    if (contentType == null) {
      throw new IllegalArgumentException();
    }
    this.contentType = contentType;
  }
}
