package lbl.akenti.servers.ResDefServer;

import java.util.StringTokenizer;
import java.util.Properties;
import java.util.Hashtable;
import java.util.Enumeration;
import java.io.FileReader;
import java.io.File;
import java.io.BufferedReader;
import java.io.IOException;


/**
  * A simple parser that parses property RootResourceName. 
  */
class RootResourceParser {

  /**
    * Returns the only instance of class RootResourceParser.
    *
    * @return the only instance of class RootResourceParser
    */
  static RootResourceParser getInstance() {
    if (parser == null) {
      synchronized (RootResourceParser.class) {
        if (parser == null) {
          parser = new RootResourceParser();
          hashtable = new Hashtable();
        }
      }
    }

    return parser;
  }

  /** 
    * Parses property RootResourceName in given file name.
    *
    * @param fileName file to be parsed.
    *
    * @return Hashtable of root resource names and their full paths.
    *
    * @Exception IOException if parameter fileName if an invalid file.
    * @Exception ConfigurationException either property RootResourceName
    *   does not have a root resource name and a full path, or the path name 
    *   is not a directory.
    */
  Hashtable parseRootResource(String fileName) 
    throws IOException, ConfigurationException {
    if (fileName == null) {
      throw new IllegalArgumentException();
    } 

    BufferedReader reader = new BufferedReader(
      new FileReader(fileName)
    );
    String line = null;
    StringTokenizer tokens = null;  
    String resourceName = null;
    String docRoot = null;
    File file = null;

    while ((line = reader.readLine()) != null) {
      if (line.startsWith("RootResourceName")) {
        tokens = new StringTokenizer(line, " "); 
       
        if (tokens.countTokens() != 3) {
          throw new ConfigurationException (
            "Must provide a <i>RootResourceName</i> property"
          );
        }

        tokens.nextToken();
        resourceName = tokens.nextToken();
        docRoot = tokens.nextToken();

        if (!docRoot.endsWith("/")) {
          docRoot += "/";
        }

        if (!docRoot.startsWith("/")) {
          throw new ConfigurationException (   
            "Must provide an absolute path for the <i>RootResourceName</i> property"
          );
        } else {
          file = new File(docRoot + resourceName);
  
          if (!file.exists()) {
            throw new ConfigurationException (  
              resourceName + " path does NOT exists"
            );
          } else if (!file.isDirectory()) {
            throw new ConfigurationException (  
              resourceName + " is not a directory"
            );
          }
        }
       
        hashtable.put(resourceName, docRoot); 
      } 
    }

    return hashtable;
  } 

  /** 
    * Returns if parameter rootName is a RootResourceName.
    *
    * @param rootName to be checked if it's a RootResourceName.
    *
    * @return true if rootName is a RootResourceName, false otherwise.
    */
  boolean hasRootResource(String rootName) {
    if (rootName == null) {
      throw new IllegalArgumentException();
    } 

    return hashtable.containsKey(rootName);
  }

  /**
    * Returns the full path for this RootResourceName.
    *
    * @param rootName path of this RootResourceName to be retrieved.
    *
    * @return full path of RootResourceName rootName.
    */
  String getRootFullPath(String rootName) {
    if (rootName == null) {
      throw new IllegalArgumentException();
    } 

    if (hasRootResource(rootName)) {
      return (String)hashtable.get(rootName);
    } 

   return null;
  }

  /**
    * Returns an Enumeration of RootResourceName.
    *
    * @return RootResourceName. 
    */
  Enumeration getRootResources() {
    return hashtable.keys();
  }

  private static RootResourceParser parser;
  private static Hashtable hashtable;
}
 
