/*
 * akenti.java
 *
 * Created on June 24, 2004, 11:10 AM
 */

package lbl.akenti.tools;

import java.util.ArrayList;
import java.lang.System;

/**
 * Recognized commands are:
 * Server side tools:
 *   CheckAccess             (CA)  - checks a user's access to a resource.
 *   CheckPolicy             (CP)  - checks a policy.
 *   ResourceBrowse          (RB)  - browse a resource
 *   CreateResource          (CR)  - creates a resource.
 *   DeleteResource          (DR)  - deletes a resource.
 *   UploadPolCert           (UP)  - uploads a policy certificate.
 *   DeletePolCert           (DP)  - deletes a policy certificate.
 * Remote tools:
 *   RemoteCheckAccess       (RCA) - checks a user's access to a remote resource.
 *   RemoteCheckPolicy       (RCP) - checks a remote policy.
 *   RemoteResourceBrowse    (RRB) - browses a remote resource.
 *   RemoteCreateResource    (RCR) - creates a remote resource.
 *   RemoteDeleteResource    (RDR) - deletes a remote resource.
 *   RemoteUploadPolCert     (RUP) - uploads a remote policy certificate.
 *   RemoteDeletePolCert     (RDP) - deletes a remote policy certificate.
 * Akenti Certificate tools:
 *   CertGen                 (CG)  - generates akenti certs from templates.
 *   PrintAkentiCertTemplate (PT)  - prints akenti cert template.
 *   ConvertAkentiCert       (CO)  - converts an akenti cert from and to XML/PEM.
 *   VerifyAkentiCert        (VA)  - verifies an akenti cert.
 *   PrettyPrintAkentiCert   (PP)  - pretty prints an akenti cert.
 *
 * @author Tommaso Sciortino
 */
public class Akenti {
  
  /**
   * Picks off the first command line argument, uses it to choose the
   * appropriate tool and passes the remaining arguments to it.
   * @param args Command line arguments.
   */  
  public static void main(String[] args) {
    if (args.length == 0) {
      printUsage();
      System.exit(2);
    }
    
    String[] newArgs = new String[args.length-1];
    System.arraycopy(args, 1, newArgs, 0, args.length - 1);
    
    if (args[0].equals("CA") || args[0].equals("CheckAccces")) {
      CheckAccess.main(newArgs);
    } else if (args[0].equals("CP") || args[0].equals("CheckPolicy")) {
      CheckPolicy.main(newArgs);
    } else if (args[0].equals("RB") || args[0].equals("ResourceBrowse")) {
      ResourceBrowse.main(newArgs);
    } else if (args[0].equals("CR") || args[0].equals("CreateResource")) {
      CreateResource.main(newArgs);
    } else if (args[0].equals("DR") || args[0].equals("DeleteResource")) {
      DeleteResource.main(newArgs);
    } else if (args[0].equals("UP") || args[0].equals("UploadPolCert")) {
      UploadPolCert.main(newArgs);
    } else if (args[0].equals("DP") || args[0].equals("DeletePolCert")) {
      DeletePolCert.main(newArgs);
      
    } else if (args[0].equals("RCA") || args[0].equals("RemoteCheckAccess")) {
      RemoteCheckAccess.main(newArgs);
    } else if (args[0].equals("RCP") || args[0].equals("RemoteCheckPolicy")) {
      RemoteCheckPolicy.main(newArgs);
    } else if (args[0].equals("RRB") || args[0].equals("RemoteResourceBrowse")) {
      RemoteResourceBrowse.main(newArgs);
    } else if (args[0].equals("RCR") || args[0].equals("RemoteCreateResource")) {
      RemoteCreateResource.main(newArgs);
    } else if (args[0].equals("RDR") || args[0].equals("RemoteDeleteResource")) {
      RemoteDeleteResource.main(newArgs);
    } else if (args[0].equals("RUP") || args[0].equals("RemoteUploadPolCert")) {
      RemoteUploadPolCert.main(newArgs);
    } else if (args[0].equals("RDP") || args[0].equals("RemoteDeletePolCert")) {
      RemoteDeletePolCert.main(newArgs);
      
    } else if (args[0].equals("CG") || args[0].equals("CertGen")) {
      CertGen.main(newArgs);
    } else if (args[0].equals("PT") || args[0].equals("PrintAkentiCertTemplate")) {
      PrintAkentiCertTemplate.main(newArgs);
    } else if (args[0].equals("CO") || args[0].equals("ConvertAkentiCert")) {
      ConvertAkentiCert.main(newArgs);
    } else if (args[0].equals("VA") || args[0].equals("VerifyAkentiCert")) {
      VerifyAkentiCert.main(newArgs);
    } else if (args[0].equals("PP") || args[0].equals("PrettyPrintAkentiCert")) {
      PrettyPrintAkentiCert.main(newArgs);
    } else {
      System.err.println("Command '" + args[0] + "' not recognized.");
      printUsage();
      System.exit(2);
    }
  }
  
  private static void printUsage(){
    System.err.println("Recognized commands are:");
    System.err.println("Server side tools:");
    System.err.println("  CheckAccess             (CA)  - checks a user's access to a resource.");
    System.err.println("  CheckPolicy             (CP)  - checks a policy.");
    System.err.println("  ResourceBrowse          (RB)  - browse a resource");
    System.err.println("  CreateResource          (CR)  - creates a resource.");
    System.err.println("  DeleteResource          (DR)  - deletes a resource.");
    System.err.println("  UploadPolCert           (UP)  - uploads a policy certificate.");
    System.err.println("  DeletePolCert           (DP)  - deletes a policy certificate.");
    System.err.println("Remote tools:");
    System.err.println("  RemoteCheckAccess       (RCA) - checks a user's access to a remote resource.");
    System.err.println("  RemoteCheckPolicy       (RCP) - checks a remote policy.");
    System.err.println("  RemoteResourceBrowse    (RRB) - browses a remote resource.");
    System.err.println("  RemoteCreateResource    (RCR) - creates a remote resource.");
    System.err.println("  RemoteDeleteResource    (RDR) - deletes a remote resource.");
    System.err.println("  RemoteUploadPolCert     (RUP) - uploads a remote policy certificate.");
    System.err.println("  RemoteDeletePolCert     (RUP) - deletes a remote policy certificate.");
    System.err.println("Akenti Certificate tools:");
    System.err.println("  CertGen                 (CG)  - generates akenti certs from templates.");
    System.err.println("  PrintAkentiCertTemplate (PT)  - prints akenti cert template.");
    System.err.println("  ConvertAkentiCert       (CO)  - converts an akenti cert from and to XML/PEM.");
    System.err.println("  VerifyAkentiCert        (VA)  - verifies an akenti cert.");
    System.err.println("  PrettyPrintAkentiCert   (PP)  - pretty prints an akenti cert");
  }
  
}
