/*
 * ConvertAkentiCert.java
 *
 * Created on July 1, 2004, 3:39 PM
 */

package lbl.akenti.tools;

import java.io.IOException;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.util.ArgumentParser;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.security.PrivateKey;
import java.security.Security;
import java.util.Properties;

/**
 * example: ConvertAkentiCert -a mycertificate -i "PEM" -o "XML"
 * options:
 * -a arg      - akenti certificate
 * -o arg      - out format PEM or XML
 * [-i arg]    - in format PEM or XML
 * [-v]        - verbose
 * [-h]        - prints this message
 *
 * @author Tommaso Sciortino
 */
public class ConvertAkentiCert {
  
  /**
   * Use this program to convert an akenti certificate
   * from/to PEM/XML formats.
   * @param args Command line argument.
   */  
  public static void main(String[] args) {
    String inFile = null;
    String inFormat = null;
    String outFormat = null;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      parser.parseArgumentTokens(new char[] {'a','i','o', 'v', 'h'} );
      
      inFile = parser.getOptionArg('a');
      inFormat = parser.getOptionArg('i');
      outFormat = parser.getOptionArg('o');
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (inFile == null || inFile.equals("")) {
        System.err.println("You must specify a certificate");
        System.exit(2);
      } else if (outFormat  == null || outFormat.equals("")) {
        System.err.println("You must specify an outFormat");
        System.exit(2);
      } else if (inFormat != null && !inFormat.equals("XML") && !inFormat.equals("PEM")) {
        System.err.println("inFormat must be either 'XML' or 'PEM'");
        System.exit(2);
      } else if (!outFormat.equals("XML") && inFormat != null && !inFormat.equals("PEM")) {
        System.err.println("outFormat must be either 'XML' or 'PEM'");
        System.exit(2);
      }
      
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      
      if (verbose) {
        System.err.println("Loading Certificate...");
        
        if (inFormat == null) {
          System.err.println("In-format not specified. Trying XML first and PEM second...");
        }
      }
      
      AkentiCertificate cert = AkentiCertificateFactory.initCertFromFile(inFormat,inFile);
      
      if (verbose) {
        System.err.println("Certificate loaded. Writing stream.");
      }
      
      AkentiCertificateFactory.writeCertToStream(outFormat,cert,System.out);
      
      System.exit(0);
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
    
  }
  /**
   * Prints the usage instructions for this program
   */  
  static void printUsage() {
    System.err.println("example: ConvertAkentiCert -a mycertificate -i PEM -o 'XML'");
    System.err.println("options:");
    System.err.println("  -a arg      - akenti certificate");
    System.err.println("  -o arg      - out format PEM or XML");
    System.err.println("  [-i arg]    - in format PEM or XML");
    System.err.println("  [-v]        - verbose");
    System.err.println("  [-h]        - prints this message");
  }
  
}
