/*
 * PrettyPrintAkentiCert.java
 *
 * Created on June 22, 2004, 1:03 PM
 */

package lbl.akenti.tools;

import java.io.IOException;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.util.ArgumentParser;


/**
 * example: PrettyPrintAkentiCert -a mycertificate
 * options:
 * -a arg  - Akenti Certificate
 * [-v]    - verbose
 * [-h]    - prints this message
 *
 * @author Tommaso Sciortino
 */
public class PrettyPrintAkentiCert {
  
  /**
   * This command line program prints a human-readable version
   * of a certificate.
   * @param args Command line arguments.
   */  
  public static void main(String[] args) {
    String certFileName = null;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
            
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }

      parser.parseArgumentTokens(new char[] {'a', 'v', 'h'} );
      
      certFileName = parser.getOptionArg('a');
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (certFileName == null || certFileName.equals("")) {
        System.err.println("Must provide a certificate");
        System.exit(2);
      }
      
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
      
    try {
      
      if (verbose) {
        System.err.println("Loading Certificate...");
      }
      AkentiCertificate cert = AkentiCertificateFactory.initCertFromFile(null,certFileName);
      
      if (verbose) {
        System.err.println("Certificate Loaded. Printing Certificate.");
      }
      
      System.out.println(cert.toText());
      
      System.exit(0);
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
  }
  
  private static void printUsage() {
    System.err.println("example: PrettyPrintAkentiCert -a mycertificate");
    System.err.println("options:");
    System.err.println("  -a arg  - Akenti Certificate");
    System.err.println("  [-v]    - verbose");
    System.err.println("  [-h]    - prints this message");
  }
  
}
