/*
 * UploadPolCert.java
 *
 * Created on July 7, 2004, 3:22 PM
 */

package lbl.akenti.tools;

import java.io.IOException;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.messaging.UploadPolicyCertificateRequest;
import lbl.akenti.messaging.UploadPolicyCertificateResponse;
import lbl.akenti.protocolHandler.LocalProtocolHandler;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.ConfFileParser;

/**
 * example: UploadPolCert -f conf.txt -p polCert
 *   -f arg          - akenti client config file
 *   -p arg          - policy certificate file
 *   [-v]            - verbose
 *   [-h]            - prints this message
 *
 * @author Tommaso Sciortino
 */
public class UploadPolCert {
  
  /**
   * Use this program to upload a policy certificate.
   * @param args Command line argument.
   */
  public static void main(String[] args) {
    String confFileName = null;
    String polCertName = null;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      parser.parseArgumentTokens(new char[] {'f', 'p', 'v', 'h'} );
      
      confFileName = parser.getOptionArg('f');
      polCertName = parser.getOptionArg('p');
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (confFileName == null || confFileName.equals("")) {
        System.err.println("Must provide a config file");
        System.exit(2);
      } else if (polCertName == null || polCertName.equals("")) {
        System.err.println("Must provide a Policy Certificate");
        System.exit(2);
      }
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      
      ConfFileParser.loadFromFile(confFileName);
      
      AkentiCertificate polCert = AkentiCertificateFactory.initCertFromFile(null,polCertName);
      if (!(polCert instanceof PolicyCertificate)) {
        System.err.println("Must provide a Certificate of type Policy");
        System.exit(2);
      }
      
      LocalProtocolHandler eng = new LocalProtocolHandler();
      UploadPolicyCertificateRequest req = new UploadPolicyCertificateRequest(LocalProtocolHandler.getID(), (PolicyCertificate) polCert);
      UploadPolicyCertificateResponse resp = eng.process(req);
      
      if (verbose) {
        System.out.println(resp.getMessage());
      }
      
      if (resp.getMessage().getCode() < 200) {
        System.exit(0);
      } else {
        System.exit(1);
      }
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
    
  }
  
  /**
   * Prints the usage instructions for this program
   */  
  static void printUsage() {
    System.err.println("example: UploadPolCert -f conf.txt -r R1/S1");
    System.err.println("options:");
    System.err.println("  -f arg      - akenti client config file");
    System.err.println("  -p arg      - policy certificate file");
    System.err.println("  [-v]        - verbose");
    System.err.println("  [-h]        - prints this message");
  }
  
}