package lbl.akenti.util;

/**
 * A collections of constants used during execution.
 * Mostly, they specify default values for constants
 * found in the configuration file.
 */
public interface AkentiConstants {
  
  /**
   * Akenti version number.
   */
  static final String CURRENT_VERSION = "1.5";
  
  /**
   * Versions of Akenti supported by these files.
   */
  static final String SUPPORTED_VERSIONS[] = { CURRENT_VERSION, null};
  
  /**
   * Default host for the Akenti Server.
   */
  static final String AkentiHost = "localhost";
  
  /**
   * Default port for the Akenti Server without SSL.
   */
  static final String AkentiPort = "8444";
  
  /**
   * Default port for the Akenti Server with SSL.
   */
  static final String AkentiPortSSL = "8443";
  
  /**
   * Default status of SSL for Akenti Server ("on" or "off").
   */
  static final String SslMode = "off";
  
  /**
   * Default name of certificate authority file (used by SSL).
   */
  static final String CertificateAuthorityFile = "";
  
  /**
   * Default value for something? (used in SSLPane in policy manager)
   */
  static final String ClientPkcs12File = "";
  
  /**
   * Default base for Cache Server relative to where the server is executed.
   */
  static final String CacheBase = ".";
  
  /**
   * Default dir for Cache Server in the CacheBase.
   */
  static final String CacheDirName = "cache";
  
  /**
   * Default port for Cache Server.
   */
  static final String CachePort = "10000";
  
  /**
   * Default policy certificate file name.
   */
  static final String PolicyFileName = "htauthority";
  
  /**
   * Default resource info file names.
   */  
  static final String ResAttrFileName = "resattrs.xml";
  
  /**
   * Default port for Resource Definition Server.
   */  
  static final String ResDefPort = "8080";
  
  /**
   * Default SSL security stance for the Resource Definition Server (either "on" or "off").
   */  
  static final String ResDefSSL = "off";
  
  /**
   * Default encoding type for Akenti certificates.
   */
  static final String AK_CERT_ENC_TYPE = "XML";
  
  /**
   * Default XML Validation stance for AkentiCertificates (either "on" or "off").
   */
  static final String XMLValidation = "off";
}