package lbl.akenti.util;

/**
 * Associates a code with a message.
 *
 * @author Abdelilah Essiari
 */

final public
class AkentiMessage {
  private int code;
  private String message;
  
  /**
   * Default constructor.
   */
  public AkentiMessage() {
    this(-1, "NO MESSAGE");
  }
  
  /**
   * Constructs this object. In general, each <CODE>code</CODE>
   * should uniquely correspond to a <CODE>message</CODE>,
   * however there are certian AkentiMessages that
   * break this style (see CheckPolicy and CheckAccess).
   * @param code the code
   * @param message the message
   */
  public AkentiMessage(int code, String message) {
    if (message == null) {
      throw new IllegalArgumentException();
    }
    
    this.code = code;
    this.message = message;
  }
  
  /**
   * Returns the <CODE>code</CODE> associated with <CODE>AkentiMessage</CODE>.
   * @return the code
   */
  public int getCode() {
    return code;
  }
  
  /**
   * Returns the <CODE>message</CODE> associated with <CODE>AkentiMessage</CODE>.
   * @return the message
   */
  public String getMessage() {
    return message;
  }
  
  /**
   * The code followed by a space followed by the message.
   * @return this object as a string
   */
  public String toString() {
    StringBuffer sb = new StringBuffer();
    
    sb.append(code);
    sb.append(' ');
    sb.append(message);
    return sb.toString();
  }
}