/*
 * CaseInsensitiveProperties.java
 *
 * Created on August 23, 2004, 10:48 AM
 */

package lbl.akenti.util;

import java.util.Enumeration;
import java.util.Properties;

/**
 * A case-insensetive version of Properties. It stores all keys in
 * lower-case and retrieves them as such. Values are unaffected.
 * @author Tommaso Sciortino
 */
public class CaseInsensitiveProperties extends Properties{
  
  /**
   * Creates a new instance of CaseInsensitiveProperties
   * @param original a non case-sensitive Properties.
   */
  public CaseInsensitiveProperties(Properties original) {
    Enumeration keys = original.keys();
    
    while (keys.hasMoreElements()) {
      String key = (String) keys.nextElement();
      String value = original.getProperty(key);
      setProperty(key, value);
    }
  }
  
  /**
   * Gets a properties value from a non case-sensitive key.
   * @param key the key
   * @return the associated value or null if there is none
   */  
  public String getProperty(String key) {
    return super.getProperty(key.toLowerCase());
  }
  
  /**
   * Gets a properties value from a non case-sensitive key.
   * @param key the key
   * @param defaultValue the value to return if the key is not present
   * @return the value if the key is found, defaultValue otherwise
   */  
  public String getProperty(String key, String defaultValue) {
    return super.getProperty(key.toLowerCase(), defaultValue.trim());
  }
  
  /**
   * Sets a property.
   * @param key the key
   * @param value the value
   * @return the value
   */  
  public Object setProperty(String key, String value) {
    return super.setProperty(key.toLowerCase(), value.trim());
  }
  
}
