package lbl.akenti.policyManager.base;

import java.util.HashMap;
import java.util.Collection;

/**
  *
  */
public class GenAttribute {
    private HashMap vals = new HashMap();
    private String name;
    private String type;

    /** 
      * Constructor.
      */
    public GenAttribute(String type, String name) {
	this.type = type;
        this.name = name;
    }

    /**
      * Returns true if the type of the attribute is X509
      */       
    public boolean isX509Attribute() {
      return type.equalsIgnoreCase("X509");
    }
    
    /**
      */
    public boolean hasName(String name) {
      return name.equalsIgnoreCase(this.name);
    }

    /**
      * Returns the name  associated with this Attribute.
      */   
    public String getName() {
	return name;
    }

    public boolean hasType(String type) {
      return type.equalsIgnoreCase(this.type);
    }

    /**
      * Returns the type of this Attribute.
      */         
    public String getType() {
	return type;
    }

    /**
      * Returns all the values associated with this Attribute.
      */
    public Object[] getValues() {
	return vals.values().toArray();
    }

    /**
      */
    public void addValues(GenX509Value[] values) {
        for (int i = 0; i < values.length; i++) {
          addValue(values[i]);
        }
    }

    public void addValues(GenValue[] values) {
        for (int i = 0; i < values.length; i++) {
          addValue(values[i]);
        }
    }

    public void addValues(Object []values) {
        for (int i = 0; i < values.length; i++) {
            if (values[i] instanceof GenValue) {
                addValue((GenValue) values[i]);
            } else {
                addValue((GenX509Value) values[i]);
            }
        }
    }

    /** 
      * Add a single GenX509Value to this attribute
      * @param value the new GenX509Value to be added
      */
    public void addValue(GenX509Value value) {
	if (!this.isX509Attribute()) {
	    throw new IllegalArgumentException();
	}
	
        String key = value.getValue();
	GenX509Value val = (GenX509Value) vals.get(key);

        if (val == null) {
          vals.put(key, value);
          return;
        }

        val.addGenCas(value.getGenCas());
    }

    /** 
     * Add a single GenValue to this attribute
     * @param value the new GenValue to be added
     */
    public void addValue(GenValue value) {
	if (this.isX509Attribute()) {
	    throw new IllegalArgumentException();
	}
	
        String key = value.getValue();
	GenValue val = (GenValue) vals.get(key);
	
        if (val == null) {
          vals.put(key, value);
          return;

        }

        val.addAkentiPrincipals(value.getAkentiPrincipals());
        val.addDirs(value.getDirs());
    }
    
    public boolean equals(Object o) {
	if (o instanceof GenAttribute) {
	    GenAttribute attr = (GenAttribute) o;

	    return name.equalsIgnoreCase(attr.name);
	} 

	return false;
    }
    
    public int hashCode() {
	return name.toLowerCase().hashCode();
    }
    
    public String toString() {
	return getName(); 
    }
    
    public String paramString() {
	StringBuffer sb = new StringBuffer();
	
	sb.append("[");
	sb.append("type = ");
	sb.append(type);
	sb.append(", name = ");
	sb.append(name);
	sb.append(",");
	
	Object []values = getValues();
	
	sb.append("number of values = ");
	sb.append(values.length);
	
	if (values.length != 0) {
	    sb.append(",");
            
            if (values[0] instanceof GenValue) {
  	      sb.append(((GenValue) values[0]).paramString());
	    
	      for (int i = 1; i < values.length; i++) {
	  	  sb.append(",");
		  sb.append(((GenValue) values[i]).paramString());
	      }
            } else {
  	      sb.append(((GenX509Value) values[0]).paramString());
	    
	      for (int i = 1; i < values.length; i++) {
	  	  sb.append(",");
		  sb.append(((GenX509Value) values[i]).paramString());
	      }
            }
	}
	
	sb.append("]");
	return sb.toString();
    }
}

