package lbl.akenti.policyManager.base;

import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;

/**
  * Wraps a policy certificate and allows for the chaining 
  * of policy certificates when dealing with a hierchical 
  * resource. 
  *
  * Recall that a policy inherits all attributes from a 
  * a parent policy with the exception of the UseCondIssuerGroups.
  * 
  **/
public class GenPolCert {

  GenPolCert parent;

  PolicyCertificate polCert;

  /**
    */
  public GenPolCert() {
    this(new PolicyCertificate());
  }

  /**
    */
  public GenPolCert(PolicyCertificate polCert) {
    this(null, polCert);
  }

  /**
    */
  public GenPolCert(GenPolCert parent, PolicyCertificate polCert) {
    if (polCert == null) {
      throw new IllegalArgumentException();
    }
    
    this.parent = parent;
    this.polCert = polCert;
  }

  /**
    */
  public boolean isRoot() {
    return parent == null;
  }

  /**
    */
  public String getResource() {
    return polCert.getResource();
  }

  /**
    */
  public CertificateAuthorityInfo[]
  getCertificateAuthorityInfos() {
    CertificateAuthorityInfo[] 
      infos = polCert.getCertificateAuthorityInfos();

    if (infos.length == 0 && parent != null) {
      infos = parent.getCertificateAuthorityInfos();
    }

    return infos;
  }

  /**
    */
  public UseCondIssuerGroup[]
  getUseCondIssuerGroups() {
    return polCert.getUseCondIssuerGroups();
  }
  
  public Directory[]
  getAttributeDirectories() {
    Directory[] attrDirs = polCert.getAttributeDirectories();
    
    if (attrDirs.length == 0 && parent != null) {
      attrDirs = parent.getAttributeDirectories(); 
    }
    
    return attrDirs;
  }
  
  /**
    */
  public long getCacheTime() {
    long cacheTime = polCert.getCacheTime();

    if (cacheTime == -1 && parent != null) {
      cacheTime = parent.getCacheTime();
    }

    return cacheTime;
  }
  
  /**
    */
  public String paramString() {
    StringBuffer sb = new StringBuffer();

    sb.append('[');
    sb.append("Parent");
    sb.append('=');
    sb.append(parent);
    sb.append(',');
    sb.append(polCert.paramString());
    sb.append(']');
    return sb.toString();
  }
}

