package lbl.akenti.policyManager.base;

import java.util.HashSet;
import java.util.Vector;

import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.Directory;

/** 
  * Class to encapsulate a SYSTEM/AKENTI/EXT_AUTH value.
  *
  */
public class GenValue {
    private String value;

    private HashSet aps = new HashSet();    
    private HashSet dirs = new HashSet();
    private Vector  args = new Vector();
    
    /**
      * Creates a new GenValue.
      */
    public GenValue(String value) {
	if (value == null) {
	    throw new IllegalArgumentException();
	}

	this.value = value;
    }
    
    public boolean hasValue(String value) {
	return value.equals(this.value);
    }

    public String getValue() {
	return value;
    }

    public void addAkentiPrincipals(AkentiPrincipal[] aps) {
        for (int i = 0; i < aps.length; ++i) {
          addAkentiPrincipal(aps[i]);
        }
    }

    public void addAkentiPrincipal(AkentiPrincipal ap) {
	aps.add(ap);
    }

    public AkentiPrincipal[] getAkentiPrincipals() {
	AkentiPrincipal [] temp = new AkentiPrincipal[aps.size()];

	aps.toArray(temp);
	return temp;
    }
    
    
    public void addDirs(Directory[] dirs) {
        for (int i = 0; i < dirs.length; ++i) {
          addDir(dirs[i]);
        }
    }

    public void addDir(Directory url) {
	dirs.add(url);
    }

    public Directory[] getDirs() {
	Directory[] temp = new Directory[dirs.size()];

	dirs.toArray(temp);
	return temp;
    }
    
    public void addArgs(String[] args) {
        for (int i = 0; i < args.length; ++i) {
          addArg(args[i]);
        }
    }

    public void addArg(String arg) {
         args.add(arg);
    }

    public String[] getArgs() {
	String[] temp = new String[args.size()];

	args.toArray(temp);
	return temp;
    }

    public String toString() {
	return value;
    }
    
    public boolean equals(Object o) {
	if (o instanceof GenValue) {
	    GenValue val = (GenValue) o;

	    return value.equals(val.value); 
	}
	
	return false;
    }
    
    public String paramString() {
	StringBuffer sb = new StringBuffer();
	
	sb.append("[value = ");
	sb.append(value);
	sb.append(",");
	
	AkentiPrincipal []aps = getAkentiPrincipals();
	
	sb.append("number of aps = ");
	sb.append(aps.length);
	
	if (aps.length != 0) {
	    sb.append(",");
	    
	    sb.append(aps[0].paramString());
	    
	    for (int i = 1; i < aps.length; i++) {
		sb.append(",");
		sb.append(aps[i].paramString());
	    }
	}
	
	sb.append("]");
	return sb.toString(); 
    }
}

